# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    AbnormalTimePeriod,
    Address,
    AddressResponse,
    AllowedAudiencesValidation,
    AllowedPrincipals,
    AnalysisData,
    AnalysisDefinition,
    ApiDefinitionInfo,
    ApiKVReference,
    ApiKVReferenceCollection,
    ApiManagementConfig,
    AppInsightsWebAppStackSettings,
    AppLogsConfiguration,
    AppRegistration,
    AppServiceCertificate,
    AppServiceCertificateCollection,
    AppServiceCertificateOrder,
    AppServiceCertificateOrderCollection,
    AppServiceCertificateOrderPatchResource,
    AppServiceCertificatePatchResource,
    AppServiceCertificateResource,
    AppServiceEnvironment,
    AppServiceEnvironmentCollection,
    AppServiceEnvironmentPatchResource,
    AppServiceEnvironmentResource,
    AppServicePlan,
    AppServicePlanCollection,
    AppServicePlanPatchResource,
    Apple,
    AppleRegistration,
    ApplicationLogsConfig,
    ApplicationStack,
    ApplicationStackCollection,
    ApplicationStackResource,
    AppserviceGithubToken,
    AppserviceGithubTokenRequest,
    ArcConfiguration,
    ArmIdWrapper,
    ArmPlan,
    AseRegion,
    AseRegionCollection,
    AseV3NetworkingConfiguration,
    AuthPlatform,
    AutoHealActions,
    AutoHealCustomAction,
    AutoHealRules,
    AutoHealTriggers,
    AzureActiveDirectory,
    AzureActiveDirectoryLogin,
    AzureActiveDirectoryRegistration,
    AzureActiveDirectoryValidation,
    AzureBlobStorageApplicationLogsConfig,
    AzureBlobStorageHttpLogsConfig,
    AzureResourceErrorInfo,
    AzureStaticWebApps,
    AzureStaticWebAppsRegistration,
    AzureStorageInfoValue,
    AzureStoragePropertyDictionaryResource,
    AzureTableStorageApplicationLogsConfig,
    BackupItem,
    BackupItemCollection,
    BackupRequest,
    BackupSchedule,
    BillingMeter,
    BillingMeterCollection,
    BlobStorageTokenStore,
    Capability,
    Certificate,
    CertificateCollection,
    CertificateDetails,
    CertificateEmail,
    CertificateOrderAction,
    CertificateOrderContact,
    CertificatePatchResource,
    ClientRegistration,
    CloningInfo,
    Configuration,
    ConnStringInfo,
    ConnStringValueTypePair,
    ConnectionStringDictionary,
    Contact,
    Container,
    ContainerApp,
    ContainerAppCollection,
    ContainerAppSecret,
    ContainerAppsConfiguration,
    ContainerCpuStatistics,
    ContainerCpuUsage,
    ContainerInfo,
    ContainerMemoryStatistics,
    ContainerNetworkInterfaceStatistics,
    ContainerResources,
    ContainerThrottlingData,
    ContentHash,
    ContentLink,
    ContinuousWebJob,
    ContinuousWebJobCollection,
    CookieExpiration,
    Correlation,
    CorsSettings,
    CsmDeploymentStatus,
    CsmDeploymentStatusCollection,
    CsmMoveResourceEnvelope,
    CsmOperationCollection,
    CsmOperationDescription,
    CsmOperationDescriptionProperties,
    CsmOperationDisplay,
    CsmPublishingCredentialsPoliciesEntity,
    CsmPublishingProfileOptions,
    CsmSlotEntity,
    CsmUsageQuota,
    CsmUsageQuotaCollection,
    CustomDnsSuffixConfiguration,
    CustomHostnameAnalysisResult,
    CustomHostnameSites,
    CustomHostnameSitesCollection,
    CustomOpenIdConnectProvider,
    CustomScaleRule,
    Dapr,
    DaprComponent,
    DaprConfig,
    DaprMetadata,
    DataProviderMetadata,
    DataSource,
    DataTableResponseColumn,
    DataTableResponseObject,
    DatabaseBackupSetting,
    DatabaseConnection,
    DatabaseConnectionCollection,
    DatabaseConnectionOverview,
    DatabaseConnectionPatchRequest,
    DefaultAuthorizationPolicy,
    DefaultErrorResponse,
    DefaultErrorResponseError,
    DefaultErrorResponseErrorDetailsItem,
    DeletedAppRestoreRequest,
    DeletedSite,
    DeletedWebAppCollection,
    Deployment,
    DeploymentCollection,
    DeploymentLocations,
    DetectorAbnormalTimePeriod,
    DetectorDefinition,
    DetectorDefinitionResource,
    DetectorInfo,
    DetectorResponse,
    DetectorResponseCollection,
    DiagnosticAnalysis,
    DiagnosticAnalysisCollection,
    DiagnosticCategory,
    DiagnosticCategoryCollection,
    DiagnosticData,
    DiagnosticDetectorCollection,
    DiagnosticDetectorResponse,
    DiagnosticMetricSample,
    DiagnosticMetricSet,
    Dimension,
    Domain,
    DomainAvailabilityCheckResult,
    DomainCollection,
    DomainControlCenterSsoRequest,
    DomainOwnershipIdentifier,
    DomainOwnershipIdentifierCollection,
    DomainPatchResource,
    DomainPurchaseConsent,
    DomainRecommendationSearchParameters,
    EnabledConfig,
    EndpointDependency,
    EndpointDetail,
    EnvironmentVar,
    ErrorEntity,
    ErrorInfo,
    ErrorProperties,
    ErrorResponse,
    Experiments,
    Expression,
    ExpressionRoot,
    ExpressionTraces,
    ExtendedLocation,
    Facebook,
    FileSystemApplicationLogsConfig,
    FileSystemHttpLogsConfig,
    FileSystemTokenStore,
    FlowAccessControlConfiguration,
    FlowAccessControlConfigurationPolicy,
    FlowEndpoints,
    FlowEndpointsConfiguration,
    ForwardProxy,
    FrontEndConfiguration,
    FunctionAppMajorVersion,
    FunctionAppMinorVersion,
    FunctionAppRuntimeSettings,
    FunctionAppRuntimes,
    FunctionAppStack,
    FunctionAppStackCollection,
    FunctionEnvelope,
    FunctionEnvelopeCollection,
    FunctionSecrets,
    GeoRegion,
    GeoRegionCollection,
    GitHub,
    GitHubActionCodeConfiguration,
    GitHubActionConfiguration,
    GitHubActionContainerConfiguration,
    GitHubActionWebAppStackSettings,
    GlobalCsmSkuDescription,
    GlobalValidation,
    Google,
    HandlerMapping,
    HostKeys,
    HostName,
    HostNameBinding,
    HostNameBindingCollection,
    HostNameSslState,
    HostingEnvironmentDeploymentInfo,
    HostingEnvironmentDiagnostics,
    HostingEnvironmentProfile,
    HttpLogsConfig,
    HttpScaleRule,
    HttpSettings,
    HttpSettingsRoutes,
    HybridConnection,
    HybridConnectionCollection,
    HybridConnectionKey,
    HybridConnectionLimits,
    Identifier,
    IdentifierCollection,
    IdentityProviders,
    InboundEnvironmentEndpoint,
    InboundEnvironmentEndpointCollection,
    Ingress,
    IpAddress,
    IpAddressRange,
    IpSecurityRestriction,
    JsonSchema,
    JwtClaimChecks,
    KeyInfo,
    KeyValuePairStringObject,
    KubeEnvironment,
    KubeEnvironmentCollection,
    KubeEnvironmentPatchResource,
    KubeEnvironmentProfile,
    LegacyMicrosoftAccount,
    LinuxJavaContainerSettings,
    LocalizableString,
    LogAnalyticsConfiguration,
    LogSpecification,
    Login,
    LoginRoutes,
    LoginScopes,
    MSDeploy,
    MSDeployLog,
    MSDeployLogEntry,
    MSDeployStatus,
    ManagedServiceIdentity,
    MetricAvailability,
    MetricSpecification,
    MigrateMySqlRequest,
    MigrateMySqlStatus,
    NameIdentifier,
    NameIdentifierCollection,
    NameValuePair,
    NetworkFeatures,
    NetworkTrace,
    Nonce,
    OpenAuthenticationAccessPolicies,
    OpenAuthenticationAccessPolicy,
    OpenAuthenticationPolicyClaim,
    OpenIdConnectClientCredential,
    OpenIdConnectConfig,
    OpenIdConnectLogin,
    OpenIdConnectRegistration,
    Operation,
    OperationResult,
    OperationResultProperties,
    OutboundEnvironmentEndpoint,
    OutboundEnvironmentEndpointCollection,
    PerfMonCounterCollection,
    PerfMonResponse,
    PerfMonSample,
    PerfMonSet,
    PremierAddOn,
    PremierAddOnOffer,
    PremierAddOnOfferCollection,
    PremierAddOnPatchResource,
    PrivateAccess,
    PrivateAccessSubnet,
    PrivateAccessVirtualNetwork,
    PrivateEndpointConnectionCollection,
    PrivateLinkConnectionApprovalRequestResource,
    PrivateLinkConnectionState,
    PrivateLinkResource,
    PrivateLinkResourceProperties,
    PrivateLinkResourcesWrapper,
    ProcessInfo,
    ProcessInfoCollection,
    ProcessModuleInfo,
    ProcessModuleInfoCollection,
    ProcessThreadInfo,
    ProcessThreadInfoCollection,
    ProxyOnlyResource,
    PublicCertificate,
    PublicCertificateCollection,
    PublishingCredentialsPoliciesCollection,
    PushSettings,
    QueryUtterancesResult,
    QueryUtterancesResults,
    QueueScaleRule,
    RampUpRule,
    Recommendation,
    RecommendationCollection,
    RecommendationRule,
    RecurrenceSchedule,
    RecurrenceScheduleOccurrence,
    RegenerateActionParameter,
    RegistryCredentials,
    ReissueCertificateOrderRequest,
    RelayServiceConnectionEntity,
    RemotePrivateEndpointConnection,
    RemotePrivateEndpointConnectionARMResource,
    Rendering,
    RenewCertificateOrderRequest,
    RepetitionIndex,
    Request,
    RequestHistory,
    RequestHistoryListResult,
    RequestHistoryProperties,
    RequestsBasedTrigger,
    Resource,
    ResourceCollection,
    ResourceConfig,
    ResourceHealthMetadata,
    ResourceHealthMetadataCollection,
    ResourceMetricAvailability,
    ResourceMetricDefinition,
    ResourceMetricDefinitionCollection,
    ResourceNameAvailability,
    ResourceNameAvailabilityRequest,
    ResourceReference,
    Response,
    ResponseMessageEnvelopeRemotePrivateEndpointConnection,
    ResponseMetaData,
    RestoreRequest,
    RetryHistory,
    Revision,
    RevisionCollection,
    RunActionCorrelation,
    RunCorrelation,
    SampleUtterance,
    Scale,
    ScaleRule,
    ScaleRuleAuth,
    Secret,
    SecretsCollection,
    ServiceSpecification,
    Site,
    SiteAuthSettings,
    SiteAuthSettingsV2,
    SiteCloneability,
    SiteCloneabilityCriterion,
    SiteConfig,
    SiteConfigPropertiesDictionary,
    SiteConfigResource,
    SiteConfigResourceCollection,
    SiteConfigurationSnapshotInfo,
    SiteConfigurationSnapshotInfoCollection,
    SiteExtensionInfo,
    SiteExtensionInfoCollection,
    SiteLimits,
    SiteLogsConfig,
    SiteMachineKey,
    SitePatchResource,
    SitePhpErrorLogFlag,
    SiteSeal,
    SiteSealRequest,
    SiteSourceControl,
    SkuCapacity,
    SkuDescription,
    SkuInfo,
    SkuInfoCollection,
    SkuInfos,
    SlotConfigNamesResource,
    SlotDifference,
    SlotDifferenceCollection,
    SlotSwapStatus,
    SlowRequestsBasedTrigger,
    Snapshot,
    SnapshotCollection,
    SnapshotRecoverySource,
    SnapshotRestoreRequest,
    Solution,
    SourceControl,
    SourceControlCollection,
    StackMajorVersion,
    StackMinorVersion,
    StampCapacity,
    StampCapacityCollection,
    StaticSiteARMResource,
    StaticSiteBasicAuthPropertiesARMResource,
    StaticSiteBasicAuthPropertiesCollection,
    StaticSiteBuildARMResource,
    StaticSiteBuildCollection,
    StaticSiteBuildProperties,
    StaticSiteCollection,
    StaticSiteCustomDomainOverviewARMResource,
    StaticSiteCustomDomainOverviewCollection,
    StaticSiteCustomDomainRequestPropertiesARMResource,
    StaticSiteDatabaseConnectionConfigurationFileOverview,
    StaticSiteFunctionOverviewARMResource,
    StaticSiteFunctionOverviewCollection,
    StaticSiteLinkedBackend,
    StaticSiteLinkedBackendARMResource,
    StaticSiteLinkedBackendsCollection,
    StaticSitePatchResource,
    StaticSiteResetPropertiesARMResource,
    StaticSiteTemplateOptions,
    StaticSiteUserARMResource,
    StaticSiteUserCollection,
    StaticSiteUserInvitationRequestResource,
    StaticSiteUserInvitationResponseResource,
    StaticSiteUserProvidedFunctionApp,
    StaticSiteUserProvidedFunctionAppARMResource,
    StaticSiteUserProvidedFunctionAppsCollection,
    StaticSiteZipDeploymentARMResource,
    StaticSitesWorkflowPreview,
    StaticSitesWorkflowPreviewRequest,
    Status,
    StatusCodesBasedTrigger,
    StatusCodesRangeBasedTrigger,
    StorageMigrationOptions,
    StorageMigrationResponse,
    StringDictionary,
    StringList,
    SubResource,
    SupportTopic,
    SwiftVirtualNetwork,
    Template,
    TldLegalAgreement,
    TldLegalAgreementCollection,
    TokenStore,
    TopLevelDomain,
    TopLevelDomainAgreementOption,
    TopLevelDomainCollection,
    TrafficWeight,
    TriggeredJobHistory,
    TriggeredJobHistoryCollection,
    TriggeredJobRun,
    TriggeredWebJob,
    TriggeredWebJobCollection,
    Twitter,
    TwitterRegistration,
    Usage,
    UsageCollection,
    User,
    UserAssignedIdentity,
    ValidateRequest,
    ValidateResponse,
    ValidateResponseError,
    VirtualApplication,
    VirtualDirectory,
    VirtualIPMapping,
    VirtualNetworkProfile,
    VnetGateway,
    VnetInfo,
    VnetInfoResource,
    VnetParameters,
    VnetRoute,
    VnetValidationFailureDetails,
    VnetValidationTestFailure,
    WebAppCollection,
    WebAppInstanceStatusCollection,
    WebAppMajorVersion,
    WebAppMinorVersion,
    WebAppRuntimeSettings,
    WebAppRuntimes,
    WebAppStack,
    WebAppStackCollection,
    WebJob,
    WebJobCollection,
    WebSiteInstanceStatus,
    WindowsJavaContainerSettings,
    WorkerPoolCollection,
    WorkerPoolResource,
    Workflow,
    WorkflowArtifacts,
    WorkflowEnvelope,
    WorkflowEnvelopeCollection,
    WorkflowEnvelopeProperties,
    WorkflowFilter,
    WorkflowHealth,
    WorkflowListResult,
    WorkflowOutputParameter,
    WorkflowParameter,
    WorkflowResource,
    WorkflowRun,
    WorkflowRunAction,
    WorkflowRunActionFilter,
    WorkflowRunActionListResult,
    WorkflowRunActionRepetitionDefinition,
    WorkflowRunActionRepetitionDefinitionCollection,
    WorkflowRunActionRepetitionProperties,
    WorkflowRunFilter,
    WorkflowRunListResult,
    WorkflowRunTrigger,
    WorkflowSku,
    WorkflowTrigger,
    WorkflowTriggerCallbackUrl,
    WorkflowTriggerFilter,
    WorkflowTriggerHistory,
    WorkflowTriggerHistoryFilter,
    WorkflowTriggerHistoryListResult,
    WorkflowTriggerListCallbackUrlQueries,
    WorkflowTriggerListResult,
    WorkflowTriggerRecurrence,
    WorkflowVersion,
    WorkflowVersionListResult,
)

from ._web_site_management_client_enums import (  # type: ignore
    ActiveRevisionsMode,
    AppServicePlanRestrictions,
    AutoHealActionType,
    AzureResourceType,
    AzureStorageState,
    AzureStorageType,
    BackupItemStatus,
    BackupRestoreOperationType,
    BasicAuthName,
    BuildStatus,
    BuiltInAuthenticationProvider,
    CertificateOrderActionType,
    CertificateOrderStatus,
    CertificateProductType,
    Channels,
    CheckNameResourceTypes,
    ClientCertMode,
    CloneAbilityResult,
    ComputeModeOptions,
    ConnectionStringType,
    ContainerAppProvisioningState,
    ContinuousWebJobStatus,
    CookieExpirationConvention,
    CustomDnsSuffixProvisioningState,
    CustomDomainStatus,
    CustomHostNameDnsRecordType,
    DaprLogLevel,
    DatabaseType,
    DayOfWeek,
    DaysOfWeek,
    DefaultAction,
    DeploymentBuildStatus,
    DetectorType,
    DnsType,
    DnsVerificationTestResult,
    DomainStatus,
    DomainType,
    EnterpriseGradeCdnStatus,
    ForwardProxyConvention,
    FrequencyUnit,
    FrontEndServiceType,
    FtpsState,
    HostNameType,
    HostType,
    HostingEnvironmentStatus,
    InAvailabilityReasonType,
    IngressTransportMethod,
    InsightStatus,
    IpFilterTag,
    IssueType,
    KeyType,
    KeyVaultSecretStatus,
    Kind,
    KubeEnvironmentProvisioningState,
    LoadBalancingMode,
    LogLevel,
    MSDeployLogEntryType,
    MSDeployProvisioningState,
    ManagedPipelineMode,
    ManagedServiceIdentityType,
    MySqlMigrationType,
    NotificationLevel,
    OpenAuthenticationProviderType,
    OperationStatus,
    ParameterType,
    ProviderOsTypeSelected,
    ProviderStackOsType,
    ProvisioningState,
    PublicCertificateLocation,
    PublishingProfileFormat,
    RecurrenceFrequency,
    RedundancyMode,
    RenderingType,
    ResolveStatus,
    ResourceNotRenewableReason,
    ResourceScopeType,
    RevisionHealthState,
    RevisionProvisioningState,
    RouteType,
    ScmType,
    SiteAvailabilityState,
    SiteExtensionType,
    SiteLoadBalancing,
    SiteRuntimeState,
    SkuName,
    SolutionType,
    SslState,
    StackPreferredOs,
    StagingEnvironmentPolicy,
    StatusOptions,
    StorageType,
    SupportedTlsVersions,
    TlsCipherSuites,
    TriggerTypes,
    TriggeredWebJobStatus,
    UnauthenticatedClientAction,
    UnauthenticatedClientActionV2,
    UpgradeAvailability,
    UpgradePreference,
    UsageState,
    ValidateResourceTypes,
    WebJobType,
    WorkerSizeOptions,
    WorkflowHealthState,
    WorkflowProvisioningState,
    WorkflowSkuName,
    WorkflowState,
    WorkflowStatus,
    WorkflowTriggerProvisioningState,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AbnormalTimePeriod",
    "Address",
    "AddressResponse",
    "AllowedAudiencesValidation",
    "AllowedPrincipals",
    "AnalysisData",
    "AnalysisDefinition",
    "ApiDefinitionInfo",
    "ApiKVReference",
    "ApiKVReferenceCollection",
    "ApiManagementConfig",
    "AppInsightsWebAppStackSettings",
    "AppLogsConfiguration",
    "AppRegistration",
    "AppServiceCertificate",
    "AppServiceCertificateCollection",
    "AppServiceCertificateOrder",
    "AppServiceCertificateOrderCollection",
    "AppServiceCertificateOrderPatchResource",
    "AppServiceCertificatePatchResource",
    "AppServiceCertificateResource",
    "AppServiceEnvironment",
    "AppServiceEnvironmentCollection",
    "AppServiceEnvironmentPatchResource",
    "AppServiceEnvironmentResource",
    "AppServicePlan",
    "AppServicePlanCollection",
    "AppServicePlanPatchResource",
    "Apple",
    "AppleRegistration",
    "ApplicationLogsConfig",
    "ApplicationStack",
    "ApplicationStackCollection",
    "ApplicationStackResource",
    "AppserviceGithubToken",
    "AppserviceGithubTokenRequest",
    "ArcConfiguration",
    "ArmIdWrapper",
    "ArmPlan",
    "AseRegion",
    "AseRegionCollection",
    "AseV3NetworkingConfiguration",
    "AuthPlatform",
    "AutoHealActions",
    "AutoHealCustomAction",
    "AutoHealRules",
    "AutoHealTriggers",
    "AzureActiveDirectory",
    "AzureActiveDirectoryLogin",
    "AzureActiveDirectoryRegistration",
    "AzureActiveDirectoryValidation",
    "AzureBlobStorageApplicationLogsConfig",
    "AzureBlobStorageHttpLogsConfig",
    "AzureResourceErrorInfo",
    "AzureStaticWebApps",
    "AzureStaticWebAppsRegistration",
    "AzureStorageInfoValue",
    "AzureStoragePropertyDictionaryResource",
    "AzureTableStorageApplicationLogsConfig",
    "BackupItem",
    "BackupItemCollection",
    "BackupRequest",
    "BackupSchedule",
    "BillingMeter",
    "BillingMeterCollection",
    "BlobStorageTokenStore",
    "Capability",
    "Certificate",
    "CertificateCollection",
    "CertificateDetails",
    "CertificateEmail",
    "CertificateOrderAction",
    "CertificateOrderContact",
    "CertificatePatchResource",
    "ClientRegistration",
    "CloningInfo",
    "Configuration",
    "ConnStringInfo",
    "ConnStringValueTypePair",
    "ConnectionStringDictionary",
    "Contact",
    "Container",
    "ContainerApp",
    "ContainerAppCollection",
    "ContainerAppSecret",
    "ContainerAppsConfiguration",
    "ContainerCpuStatistics",
    "ContainerCpuUsage",
    "ContainerInfo",
    "ContainerMemoryStatistics",
    "ContainerNetworkInterfaceStatistics",
    "ContainerResources",
    "ContainerThrottlingData",
    "ContentHash",
    "ContentLink",
    "ContinuousWebJob",
    "ContinuousWebJobCollection",
    "CookieExpiration",
    "Correlation",
    "CorsSettings",
    "CsmDeploymentStatus",
    "CsmDeploymentStatusCollection",
    "CsmMoveResourceEnvelope",
    "CsmOperationCollection",
    "CsmOperationDescription",
    "CsmOperationDescriptionProperties",
    "CsmOperationDisplay",
    "CsmPublishingCredentialsPoliciesEntity",
    "CsmPublishingProfileOptions",
    "CsmSlotEntity",
    "CsmUsageQuota",
    "CsmUsageQuotaCollection",
    "CustomDnsSuffixConfiguration",
    "CustomHostnameAnalysisResult",
    "CustomHostnameSites",
    "CustomHostnameSitesCollection",
    "CustomOpenIdConnectProvider",
    "CustomScaleRule",
    "Dapr",
    "DaprComponent",
    "DaprConfig",
    "DaprMetadata",
    "DataProviderMetadata",
    "DataSource",
    "DataTableResponseColumn",
    "DataTableResponseObject",
    "DatabaseBackupSetting",
    "DatabaseConnection",
    "DatabaseConnectionCollection",
    "DatabaseConnectionOverview",
    "DatabaseConnectionPatchRequest",
    "DefaultAuthorizationPolicy",
    "DefaultErrorResponse",
    "DefaultErrorResponseError",
    "DefaultErrorResponseErrorDetailsItem",
    "DeletedAppRestoreRequest",
    "DeletedSite",
    "DeletedWebAppCollection",
    "Deployment",
    "DeploymentCollection",
    "DeploymentLocations",
    "DetectorAbnormalTimePeriod",
    "DetectorDefinition",
    "DetectorDefinitionResource",
    "DetectorInfo",
    "DetectorResponse",
    "DetectorResponseCollection",
    "DiagnosticAnalysis",
    "DiagnosticAnalysisCollection",
    "DiagnosticCategory",
    "DiagnosticCategoryCollection",
    "DiagnosticData",
    "DiagnosticDetectorCollection",
    "DiagnosticDetectorResponse",
    "DiagnosticMetricSample",
    "DiagnosticMetricSet",
    "Dimension",
    "Domain",
    "DomainAvailabilityCheckResult",
    "DomainCollection",
    "DomainControlCenterSsoRequest",
    "DomainOwnershipIdentifier",
    "DomainOwnershipIdentifierCollection",
    "DomainPatchResource",
    "DomainPurchaseConsent",
    "DomainRecommendationSearchParameters",
    "EnabledConfig",
    "EndpointDependency",
    "EndpointDetail",
    "EnvironmentVar",
    "ErrorEntity",
    "ErrorInfo",
    "ErrorProperties",
    "ErrorResponse",
    "Experiments",
    "Expression",
    "ExpressionRoot",
    "ExpressionTraces",
    "ExtendedLocation",
    "Facebook",
    "FileSystemApplicationLogsConfig",
    "FileSystemHttpLogsConfig",
    "FileSystemTokenStore",
    "FlowAccessControlConfiguration",
    "FlowAccessControlConfigurationPolicy",
    "FlowEndpoints",
    "FlowEndpointsConfiguration",
    "ForwardProxy",
    "FrontEndConfiguration",
    "FunctionAppMajorVersion",
    "FunctionAppMinorVersion",
    "FunctionAppRuntimeSettings",
    "FunctionAppRuntimes",
    "FunctionAppStack",
    "FunctionAppStackCollection",
    "FunctionEnvelope",
    "FunctionEnvelopeCollection",
    "FunctionSecrets",
    "GeoRegion",
    "GeoRegionCollection",
    "GitHub",
    "GitHubActionCodeConfiguration",
    "GitHubActionConfiguration",
    "GitHubActionContainerConfiguration",
    "GitHubActionWebAppStackSettings",
    "GlobalCsmSkuDescription",
    "GlobalValidation",
    "Google",
    "HandlerMapping",
    "HostKeys",
    "HostName",
    "HostNameBinding",
    "HostNameBindingCollection",
    "HostNameSslState",
    "HostingEnvironmentDeploymentInfo",
    "HostingEnvironmentDiagnostics",
    "HostingEnvironmentProfile",
    "HttpLogsConfig",
    "HttpScaleRule",
    "HttpSettings",
    "HttpSettingsRoutes",
    "HybridConnection",
    "HybridConnectionCollection",
    "HybridConnectionKey",
    "HybridConnectionLimits",
    "Identifier",
    "IdentifierCollection",
    "IdentityProviders",
    "InboundEnvironmentEndpoint",
    "InboundEnvironmentEndpointCollection",
    "Ingress",
    "IpAddress",
    "IpAddressRange",
    "IpSecurityRestriction",
    "JsonSchema",
    "JwtClaimChecks",
    "KeyInfo",
    "KeyValuePairStringObject",
    "KubeEnvironment",
    "KubeEnvironmentCollection",
    "KubeEnvironmentPatchResource",
    "KubeEnvironmentProfile",
    "LegacyMicrosoftAccount",
    "LinuxJavaContainerSettings",
    "LocalizableString",
    "LogAnalyticsConfiguration",
    "LogSpecification",
    "Login",
    "LoginRoutes",
    "LoginScopes",
    "MSDeploy",
    "MSDeployLog",
    "MSDeployLogEntry",
    "MSDeployStatus",
    "ManagedServiceIdentity",
    "MetricAvailability",
    "MetricSpecification",
    "MigrateMySqlRequest",
    "MigrateMySqlStatus",
    "NameIdentifier",
    "NameIdentifierCollection",
    "NameValuePair",
    "NetworkFeatures",
    "NetworkTrace",
    "Nonce",
    "OpenAuthenticationAccessPolicies",
    "OpenAuthenticationAccessPolicy",
    "OpenAuthenticationPolicyClaim",
    "OpenIdConnectClientCredential",
    "OpenIdConnectConfig",
    "OpenIdConnectLogin",
    "OpenIdConnectRegistration",
    "Operation",
    "OperationResult",
    "OperationResultProperties",
    "OutboundEnvironmentEndpoint",
    "OutboundEnvironmentEndpointCollection",
    "PerfMonCounterCollection",
    "PerfMonResponse",
    "PerfMonSample",
    "PerfMonSet",
    "PremierAddOn",
    "PremierAddOnOffer",
    "PremierAddOnOfferCollection",
    "PremierAddOnPatchResource",
    "PrivateAccess",
    "PrivateAccessSubnet",
    "PrivateAccessVirtualNetwork",
    "PrivateEndpointConnectionCollection",
    "PrivateLinkConnectionApprovalRequestResource",
    "PrivateLinkConnectionState",
    "PrivateLinkResource",
    "PrivateLinkResourceProperties",
    "PrivateLinkResourcesWrapper",
    "ProcessInfo",
    "ProcessInfoCollection",
    "ProcessModuleInfo",
    "ProcessModuleInfoCollection",
    "ProcessThreadInfo",
    "ProcessThreadInfoCollection",
    "ProxyOnlyResource",
    "PublicCertificate",
    "PublicCertificateCollection",
    "PublishingCredentialsPoliciesCollection",
    "PushSettings",
    "QueryUtterancesResult",
    "QueryUtterancesResults",
    "QueueScaleRule",
    "RampUpRule",
    "Recommendation",
    "RecommendationCollection",
    "RecommendationRule",
    "RecurrenceSchedule",
    "RecurrenceScheduleOccurrence",
    "RegenerateActionParameter",
    "RegistryCredentials",
    "ReissueCertificateOrderRequest",
    "RelayServiceConnectionEntity",
    "RemotePrivateEndpointConnection",
    "RemotePrivateEndpointConnectionARMResource",
    "Rendering",
    "RenewCertificateOrderRequest",
    "RepetitionIndex",
    "Request",
    "RequestHistory",
    "RequestHistoryListResult",
    "RequestHistoryProperties",
    "RequestsBasedTrigger",
    "Resource",
    "ResourceCollection",
    "ResourceConfig",
    "ResourceHealthMetadata",
    "ResourceHealthMetadataCollection",
    "ResourceMetricAvailability",
    "ResourceMetricDefinition",
    "ResourceMetricDefinitionCollection",
    "ResourceNameAvailability",
    "ResourceNameAvailabilityRequest",
    "ResourceReference",
    "Response",
    "ResponseMessageEnvelopeRemotePrivateEndpointConnection",
    "ResponseMetaData",
    "RestoreRequest",
    "RetryHistory",
    "Revision",
    "RevisionCollection",
    "RunActionCorrelation",
    "RunCorrelation",
    "SampleUtterance",
    "Scale",
    "ScaleRule",
    "ScaleRuleAuth",
    "Secret",
    "SecretsCollection",
    "ServiceSpecification",
    "Site",
    "SiteAuthSettings",
    "SiteAuthSettingsV2",
    "SiteCloneability",
    "SiteCloneabilityCriterion",
    "SiteConfig",
    "SiteConfigPropertiesDictionary",
    "SiteConfigResource",
    "SiteConfigResourceCollection",
    "SiteConfigurationSnapshotInfo",
    "SiteConfigurationSnapshotInfoCollection",
    "SiteExtensionInfo",
    "SiteExtensionInfoCollection",
    "SiteLimits",
    "SiteLogsConfig",
    "SiteMachineKey",
    "SitePatchResource",
    "SitePhpErrorLogFlag",
    "SiteSeal",
    "SiteSealRequest",
    "SiteSourceControl",
    "SkuCapacity",
    "SkuDescription",
    "SkuInfo",
    "SkuInfoCollection",
    "SkuInfos",
    "SlotConfigNamesResource",
    "SlotDifference",
    "SlotDifferenceCollection",
    "SlotSwapStatus",
    "SlowRequestsBasedTrigger",
    "Snapshot",
    "SnapshotCollection",
    "SnapshotRecoverySource",
    "SnapshotRestoreRequest",
    "Solution",
    "SourceControl",
    "SourceControlCollection",
    "StackMajorVersion",
    "StackMinorVersion",
    "StampCapacity",
    "StampCapacityCollection",
    "StaticSiteARMResource",
    "StaticSiteBasicAuthPropertiesARMResource",
    "StaticSiteBasicAuthPropertiesCollection",
    "StaticSiteBuildARMResource",
    "StaticSiteBuildCollection",
    "StaticSiteBuildProperties",
    "StaticSiteCollection",
    "StaticSiteCustomDomainOverviewARMResource",
    "StaticSiteCustomDomainOverviewCollection",
    "StaticSiteCustomDomainRequestPropertiesARMResource",
    "StaticSiteDatabaseConnectionConfigurationFileOverview",
    "StaticSiteFunctionOverviewARMResource",
    "StaticSiteFunctionOverviewCollection",
    "StaticSiteLinkedBackend",
    "StaticSiteLinkedBackendARMResource",
    "StaticSiteLinkedBackendsCollection",
    "StaticSitePatchResource",
    "StaticSiteResetPropertiesARMResource",
    "StaticSiteTemplateOptions",
    "StaticSiteUserARMResource",
    "StaticSiteUserCollection",
    "StaticSiteUserInvitationRequestResource",
    "StaticSiteUserInvitationResponseResource",
    "StaticSiteUserProvidedFunctionApp",
    "StaticSiteUserProvidedFunctionAppARMResource",
    "StaticSiteUserProvidedFunctionAppsCollection",
    "StaticSiteZipDeploymentARMResource",
    "StaticSitesWorkflowPreview",
    "StaticSitesWorkflowPreviewRequest",
    "Status",
    "StatusCodesBasedTrigger",
    "StatusCodesRangeBasedTrigger",
    "StorageMigrationOptions",
    "StorageMigrationResponse",
    "StringDictionary",
    "StringList",
    "SubResource",
    "SupportTopic",
    "SwiftVirtualNetwork",
    "Template",
    "TldLegalAgreement",
    "TldLegalAgreementCollection",
    "TokenStore",
    "TopLevelDomain",
    "TopLevelDomainAgreementOption",
    "TopLevelDomainCollection",
    "TrafficWeight",
    "TriggeredJobHistory",
    "TriggeredJobHistoryCollection",
    "TriggeredJobRun",
    "TriggeredWebJob",
    "TriggeredWebJobCollection",
    "Twitter",
    "TwitterRegistration",
    "Usage",
    "UsageCollection",
    "User",
    "UserAssignedIdentity",
    "ValidateRequest",
    "ValidateResponse",
    "ValidateResponseError",
    "VirtualApplication",
    "VirtualDirectory",
    "VirtualIPMapping",
    "VirtualNetworkProfile",
    "VnetGateway",
    "VnetInfo",
    "VnetInfoResource",
    "VnetParameters",
    "VnetRoute",
    "VnetValidationFailureDetails",
    "VnetValidationTestFailure",
    "WebAppCollection",
    "WebAppInstanceStatusCollection",
    "WebAppMajorVersion",
    "WebAppMinorVersion",
    "WebAppRuntimeSettings",
    "WebAppRuntimes",
    "WebAppStack",
    "WebAppStackCollection",
    "WebJob",
    "WebJobCollection",
    "WebSiteInstanceStatus",
    "WindowsJavaContainerSettings",
    "WorkerPoolCollection",
    "WorkerPoolResource",
    "Workflow",
    "WorkflowArtifacts",
    "WorkflowEnvelope",
    "WorkflowEnvelopeCollection",
    "WorkflowEnvelopeProperties",
    "WorkflowFilter",
    "WorkflowHealth",
    "WorkflowListResult",
    "WorkflowOutputParameter",
    "WorkflowParameter",
    "WorkflowResource",
    "WorkflowRun",
    "WorkflowRunAction",
    "WorkflowRunActionFilter",
    "WorkflowRunActionListResult",
    "WorkflowRunActionRepetitionDefinition",
    "WorkflowRunActionRepetitionDefinitionCollection",
    "WorkflowRunActionRepetitionProperties",
    "WorkflowRunFilter",
    "WorkflowRunListResult",
    "WorkflowRunTrigger",
    "WorkflowSku",
    "WorkflowTrigger",
    "WorkflowTriggerCallbackUrl",
    "WorkflowTriggerFilter",
    "WorkflowTriggerHistory",
    "WorkflowTriggerHistoryFilter",
    "WorkflowTriggerHistoryListResult",
    "WorkflowTriggerListCallbackUrlQueries",
    "WorkflowTriggerListResult",
    "WorkflowTriggerRecurrence",
    "WorkflowVersion",
    "WorkflowVersionListResult",
    "ActiveRevisionsMode",
    "AppServicePlanRestrictions",
    "AutoHealActionType",
    "AzureResourceType",
    "AzureStorageState",
    "AzureStorageType",
    "BackupItemStatus",
    "BackupRestoreOperationType",
    "BasicAuthName",
    "BuildStatus",
    "BuiltInAuthenticationProvider",
    "CertificateOrderActionType",
    "CertificateOrderStatus",
    "CertificateProductType",
    "Channels",
    "CheckNameResourceTypes",
    "ClientCertMode",
    "CloneAbilityResult",
    "ComputeModeOptions",
    "ConnectionStringType",
    "ContainerAppProvisioningState",
    "ContinuousWebJobStatus",
    "CookieExpirationConvention",
    "CustomDnsSuffixProvisioningState",
    "CustomDomainStatus",
    "CustomHostNameDnsRecordType",
    "DaprLogLevel",
    "DatabaseType",
    "DayOfWeek",
    "DaysOfWeek",
    "DefaultAction",
    "DeploymentBuildStatus",
    "DetectorType",
    "DnsType",
    "DnsVerificationTestResult",
    "DomainStatus",
    "DomainType",
    "EnterpriseGradeCdnStatus",
    "ForwardProxyConvention",
    "FrequencyUnit",
    "FrontEndServiceType",
    "FtpsState",
    "HostNameType",
    "HostType",
    "HostingEnvironmentStatus",
    "InAvailabilityReasonType",
    "IngressTransportMethod",
    "InsightStatus",
    "IpFilterTag",
    "IssueType",
    "KeyType",
    "KeyVaultSecretStatus",
    "Kind",
    "KubeEnvironmentProvisioningState",
    "LoadBalancingMode",
    "LogLevel",
    "MSDeployLogEntryType",
    "MSDeployProvisioningState",
    "ManagedPipelineMode",
    "ManagedServiceIdentityType",
    "MySqlMigrationType",
    "NotificationLevel",
    "OpenAuthenticationProviderType",
    "OperationStatus",
    "ParameterType",
    "ProviderOsTypeSelected",
    "ProviderStackOsType",
    "ProvisioningState",
    "PublicCertificateLocation",
    "PublishingProfileFormat",
    "RecurrenceFrequency",
    "RedundancyMode",
    "RenderingType",
    "ResolveStatus",
    "ResourceNotRenewableReason",
    "ResourceScopeType",
    "RevisionHealthState",
    "RevisionProvisioningState",
    "RouteType",
    "ScmType",
    "SiteAvailabilityState",
    "SiteExtensionType",
    "SiteLoadBalancing",
    "SiteRuntimeState",
    "SkuName",
    "SolutionType",
    "SslState",
    "StackPreferredOs",
    "StagingEnvironmentPolicy",
    "StatusOptions",
    "StorageType",
    "SupportedTlsVersions",
    "TlsCipherSuites",
    "TriggerTypes",
    "TriggeredWebJobStatus",
    "UnauthenticatedClientAction",
    "UnauthenticatedClientActionV2",
    "UpgradeAvailability",
    "UpgradePreference",
    "UsageState",
    "ValidateResourceTypes",
    "WebJobType",
    "WorkerSizeOptions",
    "WorkflowHealthState",
    "WorkflowProvisioningState",
    "WorkflowSkuName",
    "WorkflowState",
    "WorkflowStatus",
    "WorkflowTriggerProvisioningState",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    Given an AIProjectClient, this sample demonstrates how to get an authenticated 
    ChatCompletionsClient from the azure.ai.inference package and perform one chat completion
    operation. It also shows how to turn on local tracing.
    For more information on the azure.ai.inference package see https://pypi.org/project/azure-ai-inference/.

USAGE:
    sample_chat_completions_with_azure_ai_inference_client_and_console_tracing.py

    Before running the sample:

    pip install azure-ai-projects azure-ai-inference azure-identity

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the overview page of your
       Azure AI Foundry project.
    2) MODEL_DEPLOYMENT_NAME - The AI model deployment name, as found in your AI Foundry project.

ALTERNATIVE USAGE:
    If you want to export telemetry to OTLP endpoint (such as Aspire dashboard
    https://learn.microsoft.com/dotnet/aspire/fundamentals/dashboard/standalone?tabs=bash)
    instead of to the console, also install:

    pip install opentelemetry-exporter-otlp-proto-grpc

    And also define:
    3) OTEL_INSTRUMENTATION_GENAI_CAPTURE_MESSAGE_CONTENT - Optional. Set to `true` to trace the content of chat
       messages, which may contain personal data. False by default.    
"""

import os
import sys
from azure.identity import DefaultAzureCredential
from azure.ai.projects import AIProjectClient, enable_telemetry
from azure.ai.inference.models import UserMessage

# Enable console tracing.
# or, if you have local OTLP endpoint running, change it to
# enable_telemetry(destination="http://localhost:4317")
enable_telemetry(destination=sys.stdout)

endpoint = os.environ["PROJECT_ENDPOINT"]
model_deployment_name = os.environ["MODEL_DEPLOYMENT_NAME"]

with DefaultAzureCredential(exclude_interactive_browser_credential=False) as credential:

    with AIProjectClient(endpoint=endpoint, credential=credential) as project_client:

        with project_client.inference.get_chat_completions_client() as client:

            response = client.complete(
                model=model_deployment_name, messages=[UserMessage(content="How many feet are in a mile?")]
            )

            print(response.choices[0].message.content)



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# --------------------------------------------------------------------------

from azure.core.paging import ItemPaged
from azure.core.exceptions import HttpResponseError

import pytest


class TestPaging(object):
    def test_basic_paging(self):
        def get_next(continuation_token=None):
            """Simplify my life and return JSON and not response, but should be response."""
            if not continuation_token:
                return {"nextLink": "page2", "value": ["value1.0", "value1.1"]}
            else:
                return {"nextLink": None, "value": ["value2.0", "value2.1"]}

        def extract_data(response):
            return response["nextLink"], iter(response["value"])

        pager = ItemPaged(get_next, extract_data)
        result_iterated = list(pager)

        assert ["value1.0", "value1.1", "value2.0", "value2.1"] == result_iterated

    def test_by_page_paging(self):
        def get_next(continuation_token=None):
            """Simplify my life and return JSON and not response, but should be response."""
            if not continuation_token:
                return {"nextLink": "page2", "value": ["value1.0", "value1.1"]}
            else:
                return {"nextLink": None, "value": ["value2.0", "value2.1"]}

        def extract_data(response):
            return response["nextLink"], iter(response["value"])

        pager = ItemPaged(get_next, extract_data).by_page()
        page1 = next(pager)
        assert list(page1) == ["value1.0", "value1.1"]

        page2 = next(pager)
        assert list(page2) == ["value2.0", "value2.1"]

        with pytest.raises(StopIteration):
            next(pager)

    def test_advance_paging(self):
        def get_next(continuation_token=None):
            """Simplify my life and return JSON and not response, but should be response."""
            if not continuation_token:
                return {"nextLink": "page2", "value": ["value1.0", "value1.1"]}
            else:
                return {"nextLink": None, "value": ["value2.0", "value2.1"]}

        def extract_data(response):
            return response["nextLink"], iter(response["value"])

        pager = ItemPaged(get_next, extract_data)
        page1 = next(pager)
        assert page1 == "value1.0"
        page1 = next(pager)
        assert page1 == "value1.1"

        page2 = next(pager)
        assert page2 == "value2.0"
        page2 = next(pager)
        assert page2 == "value2.1"

        with pytest.raises(StopIteration):
            next(pager)

    def test_none_value(self):
        def get_next(continuation_token=None):
            return {"nextLink": None, "value": None}

        def extract_data(response):
            return response["nextLink"], iter(response["value"] or [])

        pager = ItemPaged(get_next, extract_data)
        result_iterated = list(pager)
        assert len(result_iterated) == 0

    def test_print(self):
        def get_next(continuation_token=None):
            return {"nextLink": None, "value": None}

        def extract_data(response):
            return response["nextLink"], iter(response["value"] or [])

        pager = ItemPaged(get_next, extract_data)
        output = repr(pager)
        assert output.startswith("<iterator object azure.core.paging.ItemPaged at")

    def test_paging_continue_on_error(self):
        def get_next(continuation_token=None):
            if not continuation_token:
                return {"nextLink": "foo", "value": ["bar"]}
            else:
                raise HttpResponseError()

        def extract_data(response):
            return response["nextLink"], iter(response["value"] or [])

        pager = ItemPaged(get_next, extract_data)
        assert next(pager) == "bar"
        with pytest.raises(HttpResponseError) as err:
            next(pager)
        assert err.value.continuation_token == "foo"



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to use agent operations with the
    OpenAPI tool from the Azure Agents service using a synchronous client.
    To learn more about OpenAPI specs, visit https://learn.microsoft.com/openapi

USAGE:
    python sample_agents_openapi.py

    Before running the sample:

    pip install azure-ai-agents azure-identity jsonref

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""

import os
import jsonref
from azure.ai.agents import AgentsClient
from azure.identity import DefaultAzureCredential
from azure.ai.agents.models import OpenApiTool, OpenApiAnonymousAuthDetails

weather_asset_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../assets/weather_openapi.json"))

countries_asset_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../assets/countries.json"))

agents_client = AgentsClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)
# [START create_agent_with_openapi]

with open(weather_asset_file_path, "r") as f:
    openapi_weather = jsonref.loads(f.read())

with open(countries_asset_file_path, "r") as f:
    openapi_countries = jsonref.loads(f.read())

# Create Auth object for the OpenApiTool (note that connection or managed identity auth setup requires additional setup in Azure)
auth = OpenApiAnonymousAuthDetails()

# Initialize agent OpenApi tool using the read in OpenAPI spec
openapi_tool = OpenApiTool(
    name="get_weather", spec=openapi_weather, description="Retrieve weather information for a location", auth=auth
)
openapi_tool.add_definition(
    name="get_countries", spec=openapi_countries, description="Retrieve a list of countries", auth=auth
)

# Create agent with OpenApi tool and process agent run
with agents_client:
    agent = agents_client.create_agent(
        model=os.environ["MODEL_DEPLOYMENT_NAME"],
        name="my-agent",
        instructions="You are a helpful agent",
        tools=openapi_tool.definitions,
    )
    # [END create_agent_with_openapi]

    print(f"Created agent, ID: {agent.id}")

    # Create thread for communication
    thread = agents_client.threads.create()
    print(f"Created thread, ID: {thread.id}")

    # Create message to thread
    message = agents_client.messages.create(
        thread_id=thread.id,
        role="user",
        content="What's the weather in Seattle and What is the name and population of the country that uses currency with abbreviation THB?",
    )
    print(f"Created message, ID: {message.id}")

    # Create and process agent run in thread with tools
    run = agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
    print(f"Run finished with status: {run.status}")

    if run.status == "failed":
        print(f"Run failed: {run.last_error}")

    run_steps = agents_client.run_steps.list(thread_id=thread.id, run_id=run.id)

    # Loop through each step
    for step in run_steps:
        print(f"Step {step['id']} status: {step['status']}")

        # Check if there are tool calls in the step details
        step_details = step.get("step_details", {})
        tool_calls = step_details.get("tool_calls", [])

        if tool_calls:
            print("  Tool calls:")
            for call in tool_calls:
                print(f"    Tool Call ID: {call.get('id')}")
                print(f"    Type: {call.get('type')}")

                function_details = call.get("function", {})
                if function_details:
                    print(f"    Function name: {function_details.get('name')}")
        print()  # add an extra newline between steps

    # Delete the agent when done
    agents_client.delete_agent(agent.id)
    print("Deleted agent")

    # Fetch and log all messages
    messages = agents_client.messages.list(thread_id=thread.id)
    for msg in messages:
        if msg.text_messages:
            last_text = msg.text_messages[-1]
            print(f"{msg.role}: {last_text.text.value}")



# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import pytest
import unittest
from azure.communication.identity import *


class IdentifierRawIdTest(unittest.TestCase):
    def test_raw_id(self):
        _assert_raw_id(
            CommunicationUserIdentifier(
                id="8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
            "8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            CommunicationUserIdentifier(
                id="8:gcch-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
            "8:gcch-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(CommunicationUserIdentifier(id="someFutureFormat"), "someFutureFormat")
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130"),
            "8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"),
            "8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD"),
            "8:dod:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"),
            "8:gcch:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", is_anonymous=False),
            "8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", is_anonymous=True),
            "8:teamsvisitor:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsUserIdentifier(
                user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
                raw_id="8:orgid:legacyFormat",
            ),
            "8:orgid:legacyFormat",
        )
        _assert_raw_id(
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130"),
            "28:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"),
            "28:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD"),
            "28:dod:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"),
            "28:gcch:45ab2481-1c1c-4005-be24-0ffb879b1130",
        )
        _assert_raw_id(PhoneNumberIdentifier(value="+112345556789"), "4:+112345556789")
        _assert_raw_id(PhoneNumberIdentifier(value="112345556789"), "4:112345556789")
        _assert_raw_id(
            PhoneNumberIdentifier(value="+112345556789", raw_id="4:otherFormat"),
            "4:otherFormat",
        )
        _assert_raw_id(
            PhoneNumberIdentifier(value="otherFormat", raw_id="4:207ffef6-9444-41fb-92ab-20eacaae2768"),
            "4:207ffef6-9444-41fb-92ab-20eacaae2768",
        )
        # cspell:disable
        _assert_raw_id(
            PhoneNumberIdentifier(
                value="otherFormat",
                raw_id="4:207ffef6-9444-41fb-92ab-20eacaae2768_207ffef6-9444-41fb-92ab-20eacaae2768",
            ),
            "4:207ffef6-9444-41fb-92ab-20eacaae2768_207ffef6-9444-41fb-92ab-20eacaae2768",
        )
        _assert_raw_id(
            PhoneNumberIdentifier(
                value="otherFormat",
                raw_id="4:+112345556789_207ffef6-9444-41fb-92ab-20eacaae2768",
            ),
            "4:+112345556789_207ffef6-9444-41fb-92ab-20eacaae2768",
        )

    def test_identifier_from_raw_id(self):
        _assert_communication_identifier(
            "8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
            CommunicationUserIdentifier(
                id="8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
        )
        _assert_communication_identifier(
            "8:spool:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
            CommunicationUserIdentifier(
                id="8:spool:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
        )
        _assert_communication_identifier(
            "8:dod-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
            CommunicationUserIdentifier(
                id="8:dod-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
        )
        _assert_communication_identifier(
            "8:gcch-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130",
            CommunicationUserIdentifier(
                id="8:gcch-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
            ),
        )
        _assert_communication_identifier("8:acs:something", CommunicationUserIdentifier(id="8:acs:something"))
        _assert_communication_identifier(
            "8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130"),
        )
        _assert_communication_identifier(
            "8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsUserIdentifier(
                user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
                cloud="PUBLIC",
                is_anonymous=False,
            ),
        )
        _assert_communication_identifier(
            "8:dod:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsUserIdentifier(
                user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
                cloud="DOD",
                is_anonymous=False,
            ),
        )
        _assert_communication_identifier(
            "8:gcch:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsUserIdentifier(
                user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
                cloud="GCCH",
                is_anonymous=False,
            ),
        )
        _assert_communication_identifier(
            "8:teamsvisitor:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", is_anonymous=True),
        )
        _assert_communication_identifier(
            "8:orgid:legacyFormat",
            MicrosoftTeamsUserIdentifier(user_id="legacyFormat", cloud="PUBLIC", is_anonymous=False),
        )
        _assert_communication_identifier(
            "28:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130"),
        )
        _assert_communication_identifier(
            "28:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"),
        )
        _assert_communication_identifier(
            "28:dod:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD"),
        )
        _assert_communication_identifier(
            "28:gcch:45ab2481-1c1c-4005-be24-0ffb879b1130",
            MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"),
        )
        _assert_communication_identifier("4:+112345556789", PhoneNumberIdentifier(value="+112345556789"))
        _assert_communication_identifier("4:112345556789", PhoneNumberIdentifier(value="112345556789"))
        _assert_communication_identifier("4:otherFormat", PhoneNumberIdentifier(value="otherFormat"))
        _assert_communication_identifier(
            "4:207ffef6-9444-41fb-92ab-20eacaae2768",
            PhoneNumberIdentifier(value="207ffef6-9444-41fb-92ab-20eacaae2768"),
        )
        # cspell:disable
        _assert_communication_identifier(
            "4:207ffef6-9444-41fb-92ab-20eacaae2768_207ffef6-9444-41fb-92ab-20eacaae2768",
            PhoneNumberIdentifier(value="207ffef6-9444-41fb-92ab-20eacaae2768_207ffef6-9444-41fb-92ab-20eacaae2768"),
        )
        _assert_communication_identifier(
            "4:+112345556789_207ffef6-9444-41fb-92ab-20eacaae2768",
            PhoneNumberIdentifier(value="+112345556789_207ffef6-9444-41fb-92ab-20eacaae2768"),
        )
        # cspell:enable
        _assert_communication_identifier(
            "28:ag08-global:01234567-89ab-cdef-0123-456789abcdef",
            UnknownIdentifier(identifier="28:ag08-global:01234567-89ab-cdef-0123-456789abcdef"),
        )
        _assert_communication_identifier("", UnknownIdentifier(identifier=""))
        with pytest.raises(Exception):
            identifier_from_raw_id(None)

    def test_roundtrip(self):
        _assert_roundtrip("8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:spool:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:dod-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:gcch-acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:acs:something")
        _assert_roundtrip("8:orgid:45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:dod:45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:gcch:45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:teamsvisitor:45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("8:orgid:legacyFormat")
        _assert_roundtrip("4:+112345556789")
        _assert_roundtrip("4:112345556789")
        _assert_roundtrip("4:otherFormat")
        _assert_roundtrip("4:207ffef6-9444-41fb-92ab-20eacaae2768")
        # cspell:disable
        _assert_roundtrip("4:207ffef6-9444-41fb-92ab-20eacaae2768_207ffef6-9444-41fb-92ab-20eacaae2768")
        _assert_roundtrip("4:+112345556789_207ffef6-9444-41fb-92ab-20eacaae2768")
        # cspell:enable
        _assert_roundtrip("28:45ab2481-1c1c-4005-be24-0ffb879b1130")
        _assert_roundtrip("28:gcch-global:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("28:dod-global:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("28:orgid:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("28:gcch:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("28:dod:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("28:dod:01234567-89ab-cdef-0123-456789abcdef")
        _assert_roundtrip("")

    def test_equality_based_on_raw_id(self):
        # CommunicationUserIdentifiers are equal.
        assert CommunicationUserIdentifier(
            id="8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) == CommunicationUserIdentifier(
            id="8:acs:bbbcbc1e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
        )

        # CommunicationUserIdentifiers are not equal.
        assert CommunicationUserIdentifier(
            id="8:acs:6666e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) != CommunicationUserIdentifier(
            id="8:acs:555e-9f06-482a-b5d8-20e3f26ef0cd_45ab2481-1c1c-4005-be24-0ffb879b1130"
        )

        # MicrosoftTeamsUserIdentifiers are equal.
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) == MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"
        ) == MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD"
        ) == MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"
        ) == MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=False,
        ) == MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=False,
        )
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=True,
        ) == MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=True,
        )

        # MicrosoftTeamsUserIdentifiers are not equal.
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) != MicrosoftTeamsUserIdentifier(user_id="55ab2481-1c1c-4005-be24-0ffb879b1130")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"
        ) != MicrosoftTeamsUserIdentifier(user_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD")
        assert MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=False,
        ) != MicrosoftTeamsUserIdentifier(
            user_id="45ab2481-1c1c-4005-be24-0ffb879b1130",
            cloud="GCCH",
            is_anonymous=True,
        )

        # MicrosoftTeamsAppIdentifiers are equal.
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH"
        ) == MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="GCCH")

        # MicrosoftTeamsAppIdentifiers are not equal.
        assert MicrosoftTeamsAppIdentifier(
            app_id="54ab2481-1c1c-4005-be24-0ffb879b1130"
        ) != MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130")
        assert MicrosoftTeamsAppIdentifier(
            app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="PUBLIC"
        ) != MicrosoftTeamsAppIdentifier(app_id="45ab2481-1c1c-4005-be24-0ffb879b1130", cloud="DOD")

        # PhoneNumberIdentifiers are equal.
        assert PhoneNumberIdentifier(value="+112345556789") == PhoneNumberIdentifier(value="+112345556789")

        # PhoneNumberIdentifiers are not equal.
        assert PhoneNumberIdentifier(value="+112345556789") != PhoneNumberIdentifier(value="+512345556789")

        # UnknownIdentifiers are equal.
        assert UnknownIdentifier(identifier="28:ag08-global:01234567-89ab-cdef-0123-456789abcdef") == UnknownIdentifier(
            identifier="28:ag08-global:01234567-89ab-cdef-0123-456789abcdef"
        )

        # UnknownIdentifiers are not equal.
        assert UnknownIdentifier(identifier="48:8888-global:01234567-89ab-cdef-0123-456789abcdef") != UnknownIdentifier(
            identifier="48:ag08-global:01234567-89ab-cdef-0123-456789abcdef"
        )


def _assert_raw_id(identifier, want):
    # type: (CommunicationIdentifier, str) -> None
    assert identifier.raw_id == want


def _assert_communication_identifier(raw_id, want):
    # type: (str, CommunicationIdentifier) -> None
    got = identifier_from_raw_id(raw_id)
    assert got.raw_id == want.raw_id
    assert got.kind == want.kind
    assert len(got.properties) == len(want.properties)
    for key in want.properties:
        assert key in got.properties
        assert got.properties[key] == want.properties[key]


def _assert_roundtrip(raw_id):
    # type: (str) -> None
    assert identifier_from_raw_id(raw_id).raw_id == raw_id



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import AvailabilitySetsOperations  # type: ignore
from ._operations import ProximityPlacementGroupsOperations  # type: ignore
from ._operations import DedicatedHostGroupsOperations  # type: ignore
from ._operations import DedicatedHostsOperations  # type: ignore
from ._operations import SshPublicKeysOperations  # type: ignore
from ._operations import VirtualMachineExtensionImagesOperations  # type: ignore
from ._operations import VirtualMachineExtensionsOperations  # type: ignore
from ._operations import VirtualMachineImagesOperations  # type: ignore
from ._operations import VirtualMachineImagesEdgeZoneOperations  # type: ignore
from ._operations import UsageOperations  # type: ignore
from ._operations import VirtualMachinesOperations  # type: ignore
from ._operations import VirtualMachineScaleSetsOperations  # type: ignore
from ._operations import VirtualMachineSizesOperations  # type: ignore
from ._operations import ImagesOperations  # type: ignore
from ._operations import RestorePointCollectionsOperations  # type: ignore
from ._operations import RestorePointsOperations  # type: ignore
from ._operations import CapacityReservationGroupsOperations  # type: ignore
from ._operations import CapacityReservationsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetExtensionsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetRollingUpgradesOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMExtensionsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMsOperations  # type: ignore
from ._operations import LogAnalyticsOperations  # type: ignore
from ._operations import VirtualMachineRunCommandsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMRunCommandsOperations  # type: ignore
from ._operations import ResourceSkusOperations  # type: ignore
from ._operations import GalleriesOperations  # type: ignore
from ._operations import GalleryImagesOperations  # type: ignore
from ._operations import GalleryImageVersionsOperations  # type: ignore
from ._operations import GalleryApplicationsOperations  # type: ignore
from ._operations import GalleryApplicationVersionsOperations  # type: ignore
from ._operations import GallerySharingProfileOperations  # type: ignore
from ._operations import SharedGalleriesOperations  # type: ignore
from ._operations import SharedGalleryImagesOperations  # type: ignore
from ._operations import SharedGalleryImageVersionsOperations  # type: ignore
from ._operations import CommunityGalleriesOperations  # type: ignore
from ._operations import CommunityGalleryImagesOperations  # type: ignore
from ._operations import CommunityGalleryImageVersionsOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "AvailabilitySetsOperations",
    "ProximityPlacementGroupsOperations",
    "DedicatedHostGroupsOperations",
    "DedicatedHostsOperations",
    "SshPublicKeysOperations",
    "VirtualMachineExtensionImagesOperations",
    "VirtualMachineExtensionsOperations",
    "VirtualMachineImagesOperations",
    "VirtualMachineImagesEdgeZoneOperations",
    "UsageOperations",
    "VirtualMachinesOperations",
    "VirtualMachineScaleSetsOperations",
    "VirtualMachineSizesOperations",
    "ImagesOperations",
    "RestorePointCollectionsOperations",
    "RestorePointsOperations",
    "CapacityReservationGroupsOperations",
    "CapacityReservationsOperations",
    "VirtualMachineScaleSetExtensionsOperations",
    "VirtualMachineScaleSetRollingUpgradesOperations",
    "VirtualMachineScaleSetVMExtensionsOperations",
    "VirtualMachineScaleSetVMsOperations",
    "LogAnalyticsOperations",
    "VirtualMachineRunCommandsOperations",
    "VirtualMachineScaleSetVMRunCommandsOperations",
    "ResourceSkusOperations",
    "GalleriesOperations",
    "GalleryImagesOperations",
    "GalleryImageVersionsOperations",
    "GalleryApplicationsOperations",
    "GalleryApplicationVersionsOperations",
    "GallerySharingProfileOperations",
    "SharedGalleriesOperations",
    "SharedGalleryImagesOperations",
    "SharedGalleryImageVersionsOperations",
    "CommunityGalleriesOperations",
    "CommunityGalleryImagesOperations",
    "CommunityGalleryImageVersionsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RefreshDetails(Model):
    """Fields for tracking refresh job on the share.

    :param in_progress_refresh_job_id: If a refresh share job is currently in
     progress on this share, this field indicates the ARM resource ID of that
     job. The field is empty if no job is in progress.
    :type in_progress_refresh_job_id: str
    :param last_completed_refresh_job_time_in_utc: Indicates the completed
     time for the last refresh job on this particular share, if any.This could
     be a failed job or a successful job.
    :type last_completed_refresh_job_time_in_utc: datetime
    :param error_manifest_file: Indicates the relative path of the error xml
     for the last refresh job on this particular share, if any. This could be a
     failed job or a successful job.
    :type error_manifest_file: str
    :param last_job: Indicates the id of the last refresh job on this
     particular share,if any. This could be a failed job or a successful job.
    :type last_job: str
    """

    _attribute_map = {
        'in_progress_refresh_job_id': {'key': 'inProgressRefreshJobId', 'type': 'str'},
        'last_completed_refresh_job_time_in_utc': {'key': 'lastCompletedRefreshJobTimeInUTC', 'type': 'iso-8601'},
        'error_manifest_file': {'key': 'errorManifestFile', 'type': 'str'},
        'last_job': {'key': 'lastJob', 'type': 'str'},
    }

    def __init__(self, **kwargs):
        super(RefreshDetails, self).__init__(**kwargs)
        self.in_progress_refresh_job_id = kwargs.get('in_progress_refresh_job_id', None)
        self.last_completed_refresh_job_time_in_utc = kwargs.get('last_completed_refresh_job_time_in_utc', None)
        self.error_manifest_file = kwargs.get('error_manifest_file', None)
        self.last_job = kwargs.get('last_job', None)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
DESCRIPTION:
    This sample demonstrates how to do chat completions using a synchronous client,
    with the assistance of tools. In this sample, we use a mock function tool to retrieve
    flight information in order to answer a query about the next flight between two
    cities. Make sure that the AI model you use supports tools.

    This sample assumes the AI model is hosted on a Serverless API or
    Managed Compute endpoint. For GitHub Models or Azure OpenAI endpoints,
    the client constructor needs to be modified. See package documentation:
    https://github.com/Azure/azure-sdk-for-python/blob/main/sdk/ai/azure-ai-inference/README.md#key-concepts

USAGE:
    python sample_chat_completions_with_tools.py

    Set these two environment variables before running the sample:
    1) AZURE_AI_CHAT_ENDPOINT - Your endpoint URL, in the form 
        https://<your-deployment-name>.<your-azure-region>.models.ai.azure.com
        where `your-deployment-name` is your unique AI Model deployment name, and
        `your-azure-region` is the Azure region where your model is deployed.
    2) AZURE_AI_CHAT_KEY - Your model key. Keep it secret.
"""


def sample_chat_completions_with_tools():
    import os
    import json

    try:
        endpoint = os.environ["AZURE_AI_CHAT_ENDPOINT"]
        key = os.environ["AZURE_AI_CHAT_KEY"]
    except KeyError:
        print("Missing environment variable 'AZURE_AI_CHAT_ENDPOINT' or 'AZURE_AI_CHAT_KEY'")
        print("Set them before running this sample.")
        exit()

    from azure.ai.inference import ChatCompletionsClient
    from azure.ai.inference.models import (
        AssistantMessage,
        ChatCompletionsToolDefinition,
        CompletionsFinishReason,
        FunctionDefinition,
        SystemMessage,
        ToolMessage,
        UserMessage,
    )
    from azure.core.credentials import AzureKeyCredential

    # Define a function that retrieves flight information
    def get_flight_info(origin_city: str, destination_city: str):
        """
        This is a mock function that returns information about the next
        flight between two cities.

        Parameters:
        origin_city (str): The name of the city where the flight originates.
        destination_city (str): The destination city.

        Returns:
        str: The airline name, fight number, date and time of the next flight between the cities, in JSON format.
        """
        if origin_city == "Seattle" and destination_city == "Miami":
            return json.dumps(
                {"airline": "Delta", "flight_number": "DL123", "flight_date": "May 7th, 2024", "flight_time": "10:00AM"}
            )
        return json.dumps({"error": "No flights found between the cities"})

    # Define a function 'tool' that the model can use to retrieves flight information
    flight_info = ChatCompletionsToolDefinition(
        function=FunctionDefinition(
            name="get_flight_info",
            description="Returns information about the next flight between two cities. This includes the name of the airline, flight number and the date and time of the next flight, in JSON format.",
            parameters={
                "type": "object",
                "properties": {
                    "origin_city": {
                        "type": "string",
                        "description": "The name of the city where the flight originates",
                    },
                    "destination_city": {
                        "type": "string",
                        "description": "The flight destination city",
                    },
                },
                "required": ["origin_city", "destination_city"],
            },
        )
    )

    # Create a chat completion client. Make sure you selected a model that supports tools.
    client = ChatCompletionsClient(endpoint=endpoint, credential=AzureKeyCredential(key))

    # Make a chat completions call asking for flight information, while providing a tool to handle the request
    messages = [
        SystemMessage("You an assistant that helps users find flight information."),
        UserMessage("What is the next flights from Seattle to Miami?"),
    ]

    response = client.complete(
        messages=messages,
        tools=[flight_info],
    )

    # Note that in the above call we did not specify `tool_choice`. The service defaults to a setting equivalent
    # to specifying `tool_choice=ChatCompletionsToolChoicePreset.AUTO`. Other than ChatCompletionsToolChoicePreset
    # options, you can also explicitly force the model to use a function tool by specifying:
    # tool_choice=ChatCompletionsNamedToolChoice(function=ChatCompletionsNamedToolChoiceFunction(name="get_flight_info"))

    # The model should be asking for a tool call
    if response.choices[0].finish_reason == CompletionsFinishReason.TOOL_CALLS:

        # Append the previous model response to the chat history
        messages.append(AssistantMessage(tool_calls=response.choices[0].message.tool_calls))

        # In this sample we assume only one tool call was requested
        if response.choices[0].message.tool_calls is not None and len(response.choices[0].message.tool_calls) == 1:

            tool_call = response.choices[0].message.tool_calls[0]

            # Only tools of type function are supported. Make a function call.
            function_args = json.loads(tool_call.function.arguments.replace("'", '"'))
            print(f"Calling function `{tool_call.function.name}` with arguments {function_args}.")
            callable_func = locals()[tool_call.function.name]

            function_response = callable_func(**function_args)
            print(f"Function response = {function_response}")

            # Provide the tool response to the model, by appending it to the chat history
            messages.append(ToolMessage(function_response, tool_call_id=tool_call.id))

            # With the additional tools information on hand, get another response from the model
            response = client.complete(messages=messages, tools=[flight_info])

            print(f"Model response = {response.choices[0].message.content}")


if __name__ == "__main__":
    sample_chat_completions_with_tools()



# The MIT License (MIT)
# Copyright (c) Microsoft Corporation. All rights reserved.

import os
import unittest
import uuid

import pytest

import azure.cosmos.cosmos_client as cosmos_client
import azure.cosmos.documents as documents
import test_config
from azure.cosmos.exceptions import CosmosHttpResponseError
from azure.cosmos.partition_key import PartitionKey


class _config:
    host = test_config.TestConfig.host
    master_key = test_config.TestConfig.masterKey
    connection_policy = test_config.TestConfig.connectionPolicy
    PARTITION_KEY = 'key'
    UNIQUE_PARTITION_KEY = 'uniquePartitionKey'
    FIELD = 'field'
    DOCUMENTS_COUNT = 400
    DOCS_WITH_SAME_PARTITION_KEY = 200
    docs_with_numeric_id = 0
    sum = 0


@pytest.mark.cosmosQuery
class TestAggregateQuery(unittest.TestCase):
    client: cosmos_client.CosmosClient = None

    @classmethod
    def setUpClass(cls):
        cls._all_tests = []
        cls._setup()
        cls._generate_test_configs()

    @classmethod
    def tearDownClass(cls) -> None:
        try:
            cls.created_db.delete_container(cls.created_collection.id)
        except CosmosHttpResponseError:
            pass

    @classmethod
    def _setup(cls):
        if not _config.master_key or not _config.host:
            raise Exception(
                "You must specify your Azure Cosmos account values for "
                "'masterKey' and 'host' at the top of this class to run the "
                "tests.")

        cls.client = cosmos_client.CosmosClient(_config.host, _config.master_key)
        cls.created_db = cls.client.get_database_client(test_config.TestConfig.TEST_DATABASE_ID)
        cls.created_collection = cls._create_collection(cls.created_db)
        if _config.host == "https://localhost:8081/":
            os.environ["AZURE_COSMOS_DISABLE_NON_STREAMING_ORDER_BY"] = "True"

        # test documents
        document_definitions = []

        values = [None, False, True, "abc", "cdfg", "opqrs", "ttttttt", "xyz", "oo", "ppp"]
        for value in values:
            d = {_config.PARTITION_KEY: value, 'id': str(uuid.uuid4())}
            document_definitions.append(d)

        for i in range(_config.DOCS_WITH_SAME_PARTITION_KEY):
            d = {_config.PARTITION_KEY: _config.UNIQUE_PARTITION_KEY,
                 'resourceId': i,
                 _config.FIELD: i + 1,
                 'id': str(uuid.uuid4())}
            document_definitions.append(d)

        _config.docs_with_numeric_id = \
            _config.DOCUMENTS_COUNT - len(values) - _config.DOCS_WITH_SAME_PARTITION_KEY
        for i in range(_config.docs_with_numeric_id):
            d = {_config.PARTITION_KEY: i + 1, 'id': str(uuid.uuid4())}
            document_definitions.append(d)

        _config.sum = _config.docs_with_numeric_id \
                      * (_config.docs_with_numeric_id + 1) / 2.0

        cls._insert_doc(cls.created_collection, document_definitions)

    @classmethod
    def _generate_test_configs(cls):
        aggregate_query_format = 'SELECT VALUE {}(r.{}) FROM r WHERE {}'
        aggregate_orderby_query_format = 'SELECT VALUE {}(r.{}) FROM r WHERE {} ORDER BY r.{}'
        aggregate_configs = [
            ['AVG', _config.sum / _config.docs_with_numeric_id,
             'IS_NUMBER(r.{})'.format(_config.PARTITION_KEY)],
            ['AVG', None, 'true'],
            ['COUNT', _config.DOCUMENTS_COUNT, 'true'],
            ['MAX', 'xyz', 'true'],
            ['MIN', None, 'true'],
            ['SUM', _config.sum, 'IS_NUMBER(r.{})'.format(_config.PARTITION_KEY)],
            ['SUM', None, 'true']
        ]
        for operator, expected, condition in aggregate_configs:
            cls._all_tests.append([
                '{} {}'.format(operator, condition),
                aggregate_query_format.format(operator, _config.PARTITION_KEY, condition),
                expected])
            cls._all_tests.append([
                '{} {} OrderBy'.format(operator, condition),
                aggregate_orderby_query_format.format(operator, _config.PARTITION_KEY, condition,
                                                      _config.PARTITION_KEY),
                expected])

        aggregate_single_partition_format = 'SELECT VALUE {}(r.{}) FROM r WHERE r.{} = \'{}\''
        aggregate_orderby_single_partition_format = 'SELECT {}(r.{}) FROM r WHERE r.{} = \'{}\''
        same_partiton_sum = _config.DOCS_WITH_SAME_PARTITION_KEY * (_config.DOCS_WITH_SAME_PARTITION_KEY + 1) / 2.0
        aggregate_single_partition_configs = [
            ['AVG', same_partiton_sum / _config.DOCS_WITH_SAME_PARTITION_KEY],
            ['COUNT', _config.DOCS_WITH_SAME_PARTITION_KEY],
            ['MAX', _config.DOCS_WITH_SAME_PARTITION_KEY],
            ['MIN', 1],
            ['SUM', same_partiton_sum]
        ]
        for operator, expected in aggregate_single_partition_configs:
            cls._all_tests.append([
                '{} SinglePartition {}'.format(operator, 'SELECT VALUE'),
                aggregate_single_partition_format.format(
                    operator, _config.FIELD, _config.PARTITION_KEY, _config.UNIQUE_PARTITION_KEY), expected])
            cls._all_tests.append([
                '{} SinglePartition {}'.format(operator, 'SELECT'),
                aggregate_orderby_single_partition_format.format(
                    operator, _config.FIELD, _config.PARTITION_KEY, _config.UNIQUE_PARTITION_KEY),
                Exception()])

    def test_run_all(self):
        for test_name, query, expected_result in self._all_tests:
            test_name = "test_%s" % test_name
            try:
                self._run_one(query, expected_result)
                print(test_name + ': ' + query + " PASSED")
            except Exception as e:
                print(test_name + ': ' + query + " FAILED")
                raise e

    def _run_one(self, query, expected_result):
        self._execute_query_and_validate_results(self.created_collection, query, expected_result)

    @classmethod
    def _create_collection(cls, created_db):
        # type: (Database) -> Container
        created_collection = created_db.create_container(
            id='aggregate tests collection ' + str(uuid.uuid4()),
            indexing_policy={
                'includedPaths': [
                    {
                        'path': '/',
                        'indexes': [
                            {
                                'kind': 'Range',
                                'dataType': 'Number'
                            },
                            {
                                'kind': 'Range',
                                'dataType': 'String'
                            }
                        ]
                    }
                ]
            },
            partition_key=PartitionKey(
                path='/{}'.format(_config.PARTITION_KEY),
                kind=documents.PartitionKind.Hash,
            ),
            offer_throughput=10100
        )
        return created_collection

    @classmethod
    def _insert_doc(cls, collection, document_definitions):
        # type: (Container, Dict[str, Any]) -> Dict[str, Any]
        created_docs = []
        for d in document_definitions:
            created_doc = collection.create_item(body=d)
            created_docs.append(created_doc)

        return created_docs

    def _execute_query_and_validate_results(self, collection, query, expected):
        # type: (Container, str, Dict[str, Any]) -> None

        # executes the query and validates the results against the expected results
        result_iterable = collection.query_items(
            query=query,
            enable_cross_partition_query=True
        )

        def _verify_result():
            ######################################
            # test next() behavior
            ######################################
            it = result_iterable.__iter__()

            def invokeNext():
                return next(it)

            # validate that invocations of next() produces the same results as expected
            item = invokeNext()
            self.assertEqual(item, expected)

            # after the result set is exhausted, invoking next must raise a StopIteration exception
            self.assertRaises(StopIteration, invokeNext)

            ######################################
            # test by_page() behavior
            ######################################
            page_iter = result_iterable.by_page()
            fetched_res = list(next(page_iter))
            fetched_size = len(fetched_res)

            self.assertEqual(fetched_size, 1)
            self.assertEqual(fetched_res[0], expected)

            # no more results will be returned
            with self.assertRaises(StopIteration):
                next(page_iter)

        if isinstance(expected, Exception):
            self.assertRaises(CosmosHttpResponseError, _verify_result)
        else:
            _verify_result()


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import AIProjectPreparer
from testpreparer_async import AIProjectClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAIProjectEvaluationResultsOperationsAsync(AIProjectClientTestBaseAsync):
    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_list_versions(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = client.evaluation_results.list_versions(
            name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_list_latest(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = client.evaluation_results.list_latest()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_get_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.get_version(
            name="str",
            version="str",
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_delete_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.delete_version(
            name="str",
            version="str",
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_create(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.create(
            name="str",
            body={
                "name": "str",
                "version": "str",
                "BlobUri": "str",
                "DatasetFamily": "str",
                "DatasetName": "str",
                "Metrics": {"str": 0.0},
                "ModelAssetId": "str",
                "ModelName": "str",
                "ModelVersion": "str",
                "ResultType": "str",
                "description": "str",
                "id": "str",
                "stage": "str",
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_create_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.create_version(
            name="str",
            version="str",
            body={
                "name": "str",
                "version": "str",
                "BlobUri": "str",
                "DatasetFamily": "str",
                "DatasetName": "str",
                "Metrics": {"str": 0.0},
                "ModelAssetId": "str",
                "ModelName": "str",
                "ModelVersion": "str",
                "ResultType": "str",
                "description": "str",
                "id": "str",
                "stage": "str",
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_start_pending_upload(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.start_pending_upload(
            name="str",
            version="str",
            body={"pendingUploadType": "str", "connectionName": "str", "pendingUploadId": "str"},
        )

        # please add some check logic here by yourself
        # ...



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
FILE: sample_farm_hierarchy_complete_async.py

DESCRIPTION:
    This sample demonstrates creation of the complete farm hierarchy
    - Creation of Party
    - Creation of Farm
    - Creation of Field
    - Creation of Crop and Crop Variety
    - Creation of Season and SeasonalField
    - Creation of Boundary with a multipolygon GeoJSON

USAGE:
    ```python sample_farm_hierarchy_complete_async.py```

    Set the environment variables with your own values before running the sample:
    - `AZURE_TENANT_ID`: The tenant ID of your active directory application.
    - `AZURE_CLIENT_ID`: The client ID of your active directory application.
    - `AZURE_CLIENT_SECRET`: The client secret of your active directory application.
    - `FARMBEATS_ENDPOINT`: The FarmBeats endpoint that you want to run these samples on.
"""

from azure.core.exceptions import ResourceNotFoundError
from azure.identity.aio import DefaultAzureCredential
from azure.agrifood.farming.aio import FarmBeatsClient
import os
import asyncio
from dotenv import load_dotenv
import random


async def sample_farm_hierarchy_complete_async():

    farmbeats_endpoint = os.environ['FARMBEATS_ENDPOINT']

    credential = DefaultAzureCredential()

    client = FarmBeatsClient(
        endpoint=farmbeats_endpoint,
        credential=credential
    )

    party_id = f"contoso-party-{random.randint(0,1000)}"
    party_name = "contoso-party-name"
    party_description = "contoso-party-description"
    farm_id = "contoso-farm"
    farm_name = "contoso-farm-name"
    farm_description = "contoso-farm-description"
    field_id = "contoso-field"
    field_name = "contoso-field-name"
    field_description = "contoso-field-description"
    boundary_id = "contoso-boundary"
    boundary_name = "contoso-boundary-name"
    boundary_description = "contoso-boundary-description"
    crop_id = "contoso-crop"
    crop_name = "contoso-crop-name"
    crop_description = "contoso-crop-description"
    crop_product_id = "contoso-crop-product"
    crop_product_name = "contoso-crop_product-name"
    crop_product_description = "contoso-crop-product-description"
    season_id = "contoso-season"
    season_name = "contoso-season-name"
    season_description = "contoso-season-description"
    seasonal_field_id = "contoso-seasonal_field"
    seasonal_field_name = "contoso-seasonal_field-name"
    seasonal_field_description = "contoso-seasonal_field-description"
    year = "2021"
    start_date_time = "2021-01-01T20:08:10.137Z"
    end_date_time = "2021-06-06T20:08:10.137Z"
    multi_polygon = {
        "type": "MultiPolygon",
        "coordinates": [
            [
                [
                    [-94.05807495, 44.75916947],
                    [-94.05802487, 44.7592142],
                    [-94.05798752, 44.75921875],
                    [-94.05692697, 44.75883808],
                    [-94.05697525, 44.75861334],
                    [-94.05542493, 44.75844192],
                    [-94.05537128, 44.75891045],
                    [-94.05443251, 44.75884951],
                    [-94.05086517, 44.75856001],
                    [-94.05093491, 44.75533736],
                    [-94.05389607, 44.75516594],
                    [-94.05421793, 44.75520023],
                    [-94.05447543, 44.75534879],
                    [-94.05746988, 44.75751702],
                    [-94.05795157, 44.75824385],
                    [-94.05805349, 44.75863619],
                    [-94.05807495, 44.75916947]
                ]
            ],
            [
                [
                    [-94.05802667, 44.75929136],
                    [-94.05793598, 44.7607673],
                    [-94.05693233, 44.76072738],
                    [-94.05694842, 44.76008746],
                    [-94.05727246, 44.75988264],
                    [-94.05752903, 44.75946416],
                    [-94.05760288, 44.75923042],
                    [-94.05802667, 44.75929136]
                ]
            ]
        ]
    }

    # Step 1: Create a party.
    print(
        f"Creating or updating party with Id {party_id}...", end=" ", flush=True)
    party = await client.parties.create_or_update(
        party_id=party_id,
        party={
            "name": party_name,
            "description": party_description
        }
    )
    print("Done")

    print("Details of party:")
    print("\tID:", party["id"])
    print("\tName:", party["name"])
    print("\tDescription:", party["description"])

    # Step 2: Create a farm.
    print(
        f"Creating or updating farm with Id {farm_id}...", end=" ", flush=True)
    farm = await client.farms.create_or_update(
        party_id=party_id,
        farm_id=farm_id,
        farm={
            "name": farm_name,
            "description": farm_description
        }
    )
    print("Done")

    print("Details of farm:")
    print("\tID:", farm["id"])
    print("\tName:", farm["name"])
    print("\tParty Id:", farm["partyId"])
    print("\tDescription:", farm["description"])

    # Step 3: Create a field.
    print(
        f"Creating or updating field with Id {field_id}...", end=" ", flush=True)
    field = await client.fields.create_or_update(
        party_id=party_id,
        field_id=field_id,
        field={
            "name": field_name,
            "farmId": farm_id,
            "description": field_description
        }
    )
    print("Done")

    print("Details of field:")
    print("\tID:", field["id"])
    print("\tName:", field["name"])
    print("\tParty Id:", field["partyId"])
    print("\tFarm Id:", field["farmId"])
    print("\tName:", field["name"])
    print("\tDescription:", field["description"])

    # Step 4: Create a crop.
    print(
        f"Creating or updating crop with Id {crop_id}...", end=" ", flush=True)
    crop = await client.crops.create_or_update(
        crop_id=crop_id,
        crop={
            "name": crop_name,
            "description": crop_description
        }
    )
    print("Done")

    print("Details of crop:")
    print("\tID:", crop["id"])
    print("\tName:", crop["name"])
    print("\tDescription:", crop["description"])

    # Step 5: Create a crop product.
    print(
        f"Creating or updating crop product with Id {crop_product_id}...", end=" ", flush=True)
    crop_product = await client.crop_products.create_or_update(
        crop_product_id=crop_product_id,
        crop_product={
            "name": crop_product_name,
            "description": crop_product_description
        }
    )
    print("Done")

    print("Details of crop product:")
    print("\tID:", crop_product["id"])
    print("\tName:", crop_product["name"])
    print("\tDescription:", crop_product["description"])

    # Step 6: Create a season.
    print(
        f"Creating or updating season with Id {season_id}...", end=" ", flush=True)
    season = await client.seasons.create_or_update(
        season_id=season_id,
        season={
            "name": season_name,
            "year": year,
            "startDateTime": start_date_time,
            "endDateTime": end_date_time,
            "description": season_description
        }
    )
    print("Done")

    print("Details of season:")
    print("\tID:", season["id"])
    print("\tName:", season["name"])
    print("\tDescription:", season["description"])
    print("\tYear:", season["year"])
    print("\tStart Date Time:", season["startDateTime"])
    print("\tEnd Date Time:", season["endDateTime"])

    # Step 7: Create a seasonal field.
    print(
        f"Creating or updating seasonal field with Id {seasonal_field_id}...", end=" ", flush=True)
    seasonal_field = await client.seasonal_fields.create_or_update(
        party_id=party_id,
        seasonal_field_id=seasonal_field_id,
        seasonal_field={
            "name": seasonal_field_name,
            "farmId": farm_id,
            "fieldId": field_id,
            "seasonId": season_id,
            "cropId": crop_id,
            "cropProductIds": [crop_product_id],
            "description": seasonal_field_description
        }
    )
    print("Done")

    print("Details of seasonal field:")
    print("\tID:", seasonal_field["id"])
    print("\tName:", seasonal_field["name"])
    print("\tParty Name:", seasonal_field["partyId"])
    print("\tFarm Name:", seasonal_field["farmId"])
    print("\tCrop Name:", seasonal_field["cropId"])
    print("\tSeason Name:", seasonal_field["seasonId"])
    print("\tField Name:", seasonal_field["fieldId"])
    print("\tCrop Variety Name:", seasonal_field["cropProductIds"])
    print("\tName:", seasonal_field["name"])
    print("\tDescription:", seasonal_field["description"])

    # Step 8: Create a boundary.
    try:
        print(
            f"Trying to fetch boundary with id {boundary_id}...", end=" ", flush=True)
        boundary = await client.boundaries.get(
            party_id=party_id,
            boundary_id=boundary_id
        )
        print("Boundary already exists.")
    except ResourceNotFoundError:
        print(
            f"Doesn't exist. Creating boundary...", end=" ", flush=True)
        boundary = await client.boundaries.create_or_update(
            party_id=party_id,
            boundary_id=boundary_id,
            boundary={
                "name": boundary_name,
                "geometry": multi_polygon,
                "parentId": seasonal_field_id,
                "parentType": "SeasonalField",
                "description": boundary_description
            }
        )
        print("Done")

    print("\tDetails of boundary:")
    print("\tID:", boundary["id"])
    print("\tName:", boundary["name"])
    print("\tDescription:", boundary["description"])
    print("\tParentId:", boundary["parentId"])

    await client.close()
    await credential.close()


if __name__ == "__main__":

    load_dotenv()

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    asyncio.run(sample_farm_hierarchy_complete_async())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementAuthorizationServerOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_list_by_service(self, resource_group):
        response = self.client.authorization_server.list_by_service(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2024-05-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_get_entity_tag(self, resource_group):
        response = self.client.authorization_server.get_entity_tag(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_get(self, resource_group):
        response = self.client.authorization_server.get(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_create_or_update(self, resource_group):
        response = self.client.authorization_server.create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            parameters={
                "authorizationEndpoint": "str",
                "authorizationMethods": ["str"],
                "bearerTokenSendingMethods": ["str"],
                "clientAuthenticationMethod": ["str"],
                "clientId": "str",
                "clientRegistrationEndpoint": "str",
                "clientSecret": "str",
                "defaultScope": "str",
                "description": "str",
                "displayName": "str",
                "grantTypes": ["str"],
                "id": "str",
                "name": "str",
                "resourceOwnerPassword": "str",
                "resourceOwnerUsername": "str",
                "supportState": bool,
                "tokenBodyParameters": [{"name": "str", "value": "str"}],
                "tokenEndpoint": "str",
                "type": "str",
                "useInApiDocumentation": bool,
                "useInTestConsole": bool,
            },
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_update(self, resource_group):
        response = self.client.authorization_server.update(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            if_match="str",
            parameters={
                "authorizationEndpoint": "str",
                "authorizationMethods": ["str"],
                "bearerTokenSendingMethods": ["str"],
                "clientAuthenticationMethod": ["str"],
                "clientId": "str",
                "clientRegistrationEndpoint": "str",
                "clientSecret": "str",
                "defaultScope": "str",
                "description": "str",
                "displayName": "str",
                "grantTypes": ["str"],
                "id": "str",
                "name": "str",
                "resourceOwnerPassword": "str",
                "resourceOwnerUsername": "str",
                "supportState": bool,
                "tokenBodyParameters": [{"name": "str", "value": "str"}],
                "tokenEndpoint": "str",
                "type": "str",
                "useInApiDocumentation": bool,
                "useInTestConsole": bool,
            },
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_delete(self, resource_group):
        response = self.client.authorization_server.delete(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            if_match="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_authorization_server_list_secrets(self, resource_group):
        response = self.client.authorization_server.list_secrets(
            resource_group_name=resource_group.name,
            service_name="str",
            authsid="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from ._chat_client_async import ChatClient
from ._chat_thread_client_async import ChatThreadClient
from .._shared.user_credential_async import CommunicationTokenCredential

__all__ = ["ChatClient", "ChatThreadClient", "CommunicationTokenCredential"]



# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import xml.etree.ElementTree as ET
from flask import (
    Response,
    Blueprint,
    request,
)
from .helpers import assert_with_message

xml_api = Blueprint("xml_api", __name__)


@xml_api.route("/basic", methods=["GET", "PUT"])
def basic():
    basic_body = """<?xml version='1.0' encoding='UTF-8'?>
<slideshow
        title="Sample Slide Show"
        date="Date of publication"
        author="Yours Truly">
    <slide type="all">
        <title>Wake up to WonderWidgets!</title>
    </slide>
    <slide type="all">
        <title>Overview</title>
        <item>Why WonderWidgets are great</item>
        <item></item>
        <item>Who buys WonderWidgets</item>
    </slide>
</slideshow>"""

    if request.method == "GET":
        return Response(basic_body, status=200)
    elif request.method == "PUT":
        assert_with_message("content length", str(len(request.data)), request.headers["Content-Length"])
        parsed_xml = ET.fromstring(request.data.decode("utf-8"))
        assert_with_message("tag", "slideshow", parsed_xml.tag)
        attributes = parsed_xml.attrib
        assert_with_message("title attribute", "Sample Slide Show", attributes["title"])
        assert_with_message("date attribute", "Date of publication", attributes["date"])
        assert_with_message("author attribute", "Yours Truly", attributes["author"])
        return Response(status=200)
    return Response("You have passed in method '{}' that is not 'GET' or 'PUT'".format(request.method), status=400)



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to use agent operations with code interpreter through file attachment from
    the Azure Agents service using a synchronous client.

USAGE:
    python sample_agents_with_code_interpreter_file_attachment.py

    Before running the sample:

    pip install azure-ai-agents azure-identity

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""

import os
from azure.ai.agents import AgentsClient
from azure.ai.agents.models import CodeInterpreterTool, MessageAttachment
from azure.ai.agents.models import FilePurpose, MessageRole
from azure.identity import DefaultAzureCredential
from pathlib import Path

asset_file_path = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "../assets/synthetic_500_quarterly_results.csv")
)

agents_client = AgentsClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)

with agents_client:

    # Upload a file and wait for it to be processed
    file = agents_client.files.upload_and_poll(file_path=asset_file_path, purpose=FilePurpose.AGENTS)
    print(f"Uploaded file, file ID: {file.id}")

    # [START create_agent_and_message_with_code_interpreter_file_attachment]
    # Notice that CodeInterpreter must be enabled in the agent creation,
    # otherwise the agent will not be able to see the file attachment for code interpretation
    agent = agents_client.create_agent(
        model=os.environ["MODEL_DEPLOYMENT_NAME"],
        name="my-agent",
        instructions="You are helpful agent",
        tools=CodeInterpreterTool().definitions,
    )
    print(f"Created agent, agent ID: {agent.id}")

    thread = agents_client.threads.create()
    print(f"Created thread, thread ID: {thread.id}")

    # Create an attachment
    attachment = MessageAttachment(file_id=file.id, tools=CodeInterpreterTool().definitions)

    # Create a message
    message = agents_client.messages.create(
        thread_id=thread.id,
        role="user",
        content="Could you please create bar chart in TRANSPORTATION sector for the operating profit from the uploaded csv file and provide file to me?",
        attachments=[attachment],
    )
    # [END create_agent_and_message_with_code_interpreter_file_attachment]
    print(f"Created message, message ID: {message.id}")

    run = agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
    print(f"Run finished with status: {run.status}")

    if run.status == "failed":
        # Check if you got "Rate limit is exceeded.", then you want to get more quota
        print(f"Run failed: {run.last_error}")

    agents_client.files.delete(file.id)
    print("Deleted file")

    messages = agents_client.messages.list(thread_id=thread.id)
    print(f"Messages: {messages}")

    last_msg = agents_client.messages.get_last_message_text_by_role(thread_id=thread.id, role=MessageRole.AGENT)
    if last_msg:
        print(f"Last Message: {last_msg.text.value}")

    for msg in messages:
        # Save every image file in the message
        for img in msg.image_contents:
            file_id = img.image_file.file_id
            file_name = f"{file_id}_image_file.png"
            agents_client.files.save(file_id=file_id, file_name=file_name)
            print(f"Saved image file to: {Path.cwd() / file_name}")

        # Print details of every file-path annotation
        for ann in msg.file_path_annotations:
            print("File Paths:")
            print(f"  Type: {ann.type}")
            print(f"  Text: {ann.text}")
            print(f"  File ID: {ann.file_path.file_id}")
            print(f"  Start Index: {ann.start_index}")
            print(f"  End Index: {ann.end_index}")

    agents_client.delete_agent(agent.id)
    print("Deleted agent")



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest

from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.core.credentials import AzureKeyCredential
from devtools_testutils import AzureRecordedTestCase


class TestOrchestrationAppQnaResponseAsync(AzureRecordedTestCase):

    @pytest.mark.asyncio
    async def test_orchestration_app_qna_response(self, recorded_test, conversation_creds):

        # analyze query
        client = ConversationAnalysisClient(
            conversation_creds["endpoint"],
            AzureKeyCredential(conversation_creds["key"])
        )
        async with client:
            query = "How are you?"
            result = await client.analyze_conversation(
                task={
                    "kind": "Conversation",
                    "analysisInput": {
                        "conversationItem": {
                            "participantId": "1",
                            "id": "1",
                            "modality": "text",
                            "language": "en",
                            "text": query
                        },
                        "isLoggingEnabled": False
                    },
                    "parameters": {
                        "projectName": conversation_creds["orch_project_name"],
                        "deploymentName": conversation_creds["orch_deployment_name"],
                        "verbose": True
                    }
                }
            )

            # assert - main object
            top_project = 'ChitChat-QnA'
            assert not result is None
            assert result["kind"] == "ConversationResult"
            assert result["result"]["query"] == query

            # assert - prediction type
            assert result["result"]["prediction"]["projectKind"] == "Orchestration"

            # assert - top matching project
            assert result["result"]["prediction"]["topIntent"] == top_project
            top_intent_object = result["result"]["prediction"]["intents"][top_project]
            assert top_intent_object["targetProjectKind"] == "QuestionAnswering"

            # assert intent and entities
            qna_result = top_intent_object["result"]
            answer = qna_result["answers"][0]["answer"]
            assert not answer is None
            assert qna_result["answers"][0]["confidenceScore"] >= 0



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import pytest
from unittest.mock import Mock

# module under test
from corehttp.exceptions import (
    HttpResponseError,
    SerializationError,
    DeserializationError,
)
from corehttp.rest import HttpRequest

from rest_client import MockRestClient
from utils import SYNC_TRANSPORTS


class FakeErrorOne(object):
    def __init__(self):
        self.error = Mock(message="A fake error", code="FakeErrorOne")


class FakeErrorTwo(object):
    def __init__(self):
        self.code = "FakeErrorTwo"
        self.message = "A different fake error"


class FakeHttpResponse(HttpResponseError):
    def __init__(self, response, error, *args, **kwargs):
        self.error = error
        super(FakeHttpResponse, self).__init__(self, response=response, *args, **kwargs)


class TestExceptions(object):
    def test_empty_httpresponse_error(self):
        error = HttpResponseError()
        assert str(error) == "Operation returned an invalid status 'None'"
        assert error.message == "Operation returned an invalid status 'None'"
        assert error.response is None
        assert error.reason is None
        assert error.status_code is None

    def test_message_httpresponse_error(self):
        error = HttpResponseError(message="Specific error message")
        assert str(error) == "Specific error message"
        assert error.message == "Specific error message"
        assert error.response is None
        assert error.reason is None
        assert error.status_code is None

    def test_error_continuation_token(self):
        error = HttpResponseError(message="Specific error message", continuation_token="foo")
        assert str(error) == "Specific error message"
        assert error.message == "Specific error message"
        assert error.response is None
        assert error.reason is None
        assert error.status_code is None
        assert error.continuation_token == "foo"

    @pytest.mark.parametrize("transport", SYNC_TRANSPORTS)
    def test_httpresponse_error_with_response(self, port, transport):
        request = HttpRequest("GET", url="http://localhost:{}/basic/string".format(port))
        client = MockRestClient(port, transport=transport())
        response = client.send_request(request, stream=False)
        error = HttpResponseError(response=response)
        assert error.message == "Operation returned an invalid status 'OK'"
        assert error.response is not None
        assert error.reason == "OK"
        assert isinstance(error.status_code, int)

    @pytest.mark.parametrize("transport", SYNC_TRANSPORTS)
    def test_httpresponse_error_with_model(self, port, transport):
        request = HttpRequest("GET", url="http://localhost:{}/basic/string".format(port))
        client = MockRestClient(port, transport=transport())
        response = client.send_request(request, stream=False)
        error = HttpResponseError(response=response, model=FakeErrorTwo())
        assert error.message == "Operation returned an invalid status 'OK'"
        assert error.response is not None
        assert error.reason == "OK"
        assert error.model
        assert error.model.code == "FakeErrorTwo"
        assert error.model.message == "A different fake error"
        assert isinstance(error.status_code, int)

    @pytest.mark.parametrize("transport", SYNC_TRANSPORTS)
    def test_malformed_json(self, port, transport):
        request = HttpRequest("GET", "/errors/malformed-json")
        client = MockRestClient(port, transport=transport())
        response = client.send_request(request)
        with pytest.raises(HttpResponseError) as ex:
            response.raise_for_status()
        assert (
            str(ex.value)
            == 'Operation returned an invalid status \'BAD REQUEST\'\nContent: {"code": 400, "error": {"global": ["MY-ERROR-MESSAGE-THAT-IS-COMING-FROM-THE-API"]'
        )

    @pytest.mark.parametrize("transport", SYNC_TRANSPORTS)
    def test_text(self, port, transport):
        request = HttpRequest("GET", "/errors/text")
        client = MockRestClient(port, transport=transport())
        response = client.send_request(request)
        with pytest.raises(HttpResponseError) as ex:
            response.raise_for_status()
        assert str(ex.value) == "Operation returned an invalid status 'BAD REQUEST'\nContent: I am throwing an error"


def test_serialization_error():
    message = "Oopsy bad input passed for serialization"
    error = SerializationError(message)
    with pytest.raises(SerializationError) as ex:
        raise error
    assert str(ex.value) == message

    with pytest.raises(ValueError) as ex:
        raise error
    assert str(ex.value) == message


def test_deserialization_error():
    message = "Oopsy bad input passed for serialization"
    error = DeserializationError(message)
    with pytest.raises(DeserializationError) as ex:
        raise error
    assert str(ex.value) == message

    with pytest.raises(ValueError) as ex:
        raise error
    assert str(ex.value) == message



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
from azure.cosmos.aio import CosmosClient
import azure.cosmos.exceptions as exceptions
from azure.cosmos import ThroughputProperties

import asyncio
import config

# ----------------------------------------------------------------------------------------------------------
# Prerequisites -
#
# 1. An Azure Cosmos account -
#    https://learn.microsoft.com/azure/cosmos-db/create-sql-api-python#create-a-database-account
#
# 2. Microsoft Azure Cosmos PyPi package -
#    https://pypi.python.org/pypi/azure-cosmos/
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates the basic CRUD operations on a Database resource for Azure Cosmos
#
# 1. Query for Database (QueryDatabases)
#
# 2. Create Database (CreateDatabase)
#
# 3. Get a Database by its Id property (ReadDatabase)
#
# 4. List all Database resources on an account (ReadDatabases)
#
# 5. Delete a Database given its Id property (DeleteDatabase)
# ----------------------------------------------------------------------------------------------------------

HOST = config.settings['host']
MASTER_KEY = config.settings['master_key']
DATABASE_ID = config.settings['database_id']

async def find_database(client, id):
    print('1. Query for Database')

    # Because the asynchronous client returns an asynchronous iterator object for methods that use
    # return several databases using queries, we do not need to await the function. However, attempting
    # to cast this object into a list directly will throw an error; instead, iterate over the databases
    # to populate your list using an async for loop like shown here or in the list_databases() method
    query_databases_response = client.query_databases(query={
        "query": "SELECT * FROM r WHERE r.id=@id",
        "parameters": [
            { "name":"@id", "value": id }
        ]
    })

    databases = [database async for database in query_databases_response]

    if len(databases) > 0:
        print('Database with id \'{0}\' was found'.format(id))
    else:
        print('No database with id \'{0}\' was found'. format(id))

    # Alternatively, you can directly iterate over the asynchronous iterator without building a separate
    # list if you don't need the ordering or indexing capabilities
    async for database in query_databases_response:
        print(database['id'])


async def create_database(client, id):
    print("\n2. Create Database")

    try:
        await client.create_database(id=id)
        print('Database with id \'{0}\' created'.format(id))

    except exceptions.CosmosResourceExistsError:
        print('A database with id \'{0}\' already exists'.format(id))

    print("\n2.8 Create Database - With autoscale settings")

    try:
        await client.create_database(
            id=id,
            offer_throughput=ThroughputProperties(auto_scale_max_throughput=5000, auto_scale_increment_percent=0))
        print('Database with id \'{0}\' created'.format(id))

    except exceptions.CosmosResourceExistsError:
        print('A database with id \'{0}\' already exists'.format(id))

    # Alternatively, you can also use the create_database_if_not_exists method to avoid using a try catch
    # This method attempts to read the database first, and based on the result either creates or returns
    # the existing database. Due to the additional overhead from attempting a read, it is recommended
    # to use the create_database() method if you know the database doesn't already exist.
    await client.create_database_if_not_exists(id=id)


async def read_database(client, id):
    print("\n3. Get a Database by id")

    try:
        database = client.get_database_client(id)
        await database.read()
        print('Database with id \'{0}\' was found, it\'s link is {1}'.format(id, database.database_link))

    except exceptions.CosmosResourceNotFoundError:
        print('A database with id \'{0}\' does not exist'.format(id))


async def list_databases(client):
    print("\n4. List all Databases on an account")

    print('Databases:')

    # Because the asynchronous client returns an asynchronous iterator object for methods that use
    # return several databases using queries, we do not need to await the function. However, attempting
    # to cast this object into a list directly will throw an error; instead, iterate over the databases
    # to populate your list using an async for loop like shown here or in the find_database() method
    list_databases_response = client.list_databases()
    databases = [database async for database in list_databases_response]

    if len(databases) == 0:
        return

    for database in databases:
        print(database['id'])

    # Alternatively, you can directly iterate over the asynchronous iterator without building a separate
    # list if you don't need the ordering or indexing capabilities
    async for database in list_databases_response:
        print(database['id'])


async def delete_database(client, id):
    print("\n5. Delete Database")

    try:
        await client.delete_database(id)
        print('Database with id \'{0}\' was deleted'.format(id))

    except exceptions.CosmosResourceNotFoundError:
        print('A database with id \'{0}\' does not exist'.format(id))


async def run_sample():
    async with CosmosClient(HOST, {'masterKey': MASTER_KEY}) as client:
        try:
            # query for a database
            await find_database(client, DATABASE_ID)

            # create a database
            await create_database(client, DATABASE_ID)

            # get a database using its id
            await read_database(client, DATABASE_ID)

            # list all databases on an account
            await list_databases(client)

            # delete database by id
            await delete_database(client, DATABASE_ID)

        except exceptions.CosmosHttpResponseError as e:
            print('\nrun_sample has caught an error. {0}'.format(e.message))

        finally:
            print("\nrun_sample done")

if __name__ == '__main__':
    asyncio.run(run_sample())



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from typing import Optional, Type
from types import TracebackType
from copy import deepcopy
from azure.core import AsyncPipelineClient
from azure.core.pipeline import policies
from azure.core.configuration import Configuration


class TestRestClientConfiguration(Configuration):
    def __init__(self, **kwargs):
        # type: (...) -> None
        super(TestRestClientConfiguration, self).__init__(**kwargs)

        kwargs.setdefault("sdk_moniker", "autorestswaggerbatfileservice/1.0.0b1")
        self._configure(**kwargs)

    def _configure(self, **kwargs) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get("http_logging_policy") or policies.HttpLoggingPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.AsyncRetryPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get("custom_hook_policy") or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.AsyncRedirectPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")


class AsyncMockRestClient(object):
    def __init__(self, port, **kwargs):
        self._config = TestRestClientConfiguration(**kwargs)

        self._client = AsyncPipelineClient(base_url="http://localhost:{}".format(port), config=self._config, **kwargs)

    def send_request(self, request, **kwargs):
        """Runs the network request through the client's chained policies.
        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "http://localhost:3000/helloWorld")
        <HttpRequest [GET], url: 'http://localhost:3000/helloWorld'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>
        For more information on this code flow, see https://aka.ms/azsdk/python/protocol/quickstart
        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """
        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self):
        await self._client.__aenter__()
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]] = None,
        exc_value: Optional[BaseException] = None,
        traceback: Optional[TracebackType] = None,
    ) -> None:
        await self._client.__aexit__(exc_type, exc_value, traceback)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._service_topologies_operations import ServiceTopologiesOperations
from ._services_operations import ServicesOperations
from ._service_units_operations import ServiceUnitsOperations
from ._steps_operations import StepsOperations
from ._rollouts_operations import RolloutsOperations
from ._artifact_sources_operations import ArtifactSourcesOperations
from ._operations import Operations

from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ServiceTopologiesOperations",
    "ServicesOperations",
    "ServiceUnitsOperations",
    "StepsOperations",
    "RolloutsOperations",
    "ArtifactSourcesOperations",
    "Operations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import time
import base64
from typing import Any, Awaitable, Optional, cast, TypeVar, Union

from azure.core.credentials import AccessToken, AccessTokenInfo, TokenRequestOptions
from azure.core.credentials_async import (
    AsyncTokenCredential,
    AsyncSupportsTokenInfo,
    AsyncTokenProvider,
)
from azure.core.pipeline import PipelineRequest, PipelineResponse
from azure.core.pipeline.policies import AsyncHTTPPolicy
from azure.core.pipeline.policies._authentication import (
    _BearerTokenCredentialPolicyBase,
)
from azure.core.pipeline.transport import (
    AsyncHttpResponse as LegacyAsyncHttpResponse,
    HttpRequest as LegacyHttpRequest,
)
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.utils._utils import get_running_async_lock
from ._utils import get_challenge_parameter

from .._tools_async import await_result

AsyncHTTPResponseType = TypeVar("AsyncHTTPResponseType", AsyncHttpResponse, LegacyAsyncHttpResponse)
HTTPRequestType = TypeVar("HTTPRequestType", HttpRequest, LegacyHttpRequest)


class AsyncBearerTokenCredentialPolicy(AsyncHTTPPolicy[HTTPRequestType, AsyncHTTPResponseType]):
    """Adds a bearer token Authorization header to requests.

    :param credential: The credential.
    :type credential: ~azure.core.credentials_async.AsyncTokenProvider
    :param str scopes: Lets you specify the type of access needed.
    :keyword bool enable_cae: Indicates whether to enable Continuous Access Evaluation (CAE) on all requested
        tokens. Defaults to False.
    """

    def __init__(self, credential: AsyncTokenProvider, *scopes: str, **kwargs: Any) -> None:
        super().__init__()
        self._credential = credential
        self._scopes = scopes
        self._lock_instance = None
        self._token: Optional[Union["AccessToken", "AccessTokenInfo"]] = None
        self._enable_cae: bool = kwargs.get("enable_cae", False)

    @property
    def _lock(self):
        if self._lock_instance is None:
            self._lock_instance = get_running_async_lock()
        return self._lock_instance

    async def on_request(self, request: PipelineRequest[HTTPRequestType]) -> None:
        """Adds a bearer token Authorization header to request and sends request to next policy.

        :param request: The pipeline request object to be modified.
        :type request: ~azure.core.pipeline.PipelineRequest
        :raises ~azure.core.exceptions.ServiceRequestError: If the request fails.
        """
        _BearerTokenCredentialPolicyBase._enforce_https(request)  # pylint:disable=protected-access

        if self._token is None or self._need_new_token():
            async with self._lock:
                # double check because another coroutine may have acquired a token while we waited to acquire the lock
                if self._token is None or self._need_new_token():
                    await self._request_token(*self._scopes)
        bearer_token = cast(Union[AccessToken, AccessTokenInfo], self._token).token
        request.http_request.headers["Authorization"] = "Bearer " + bearer_token

    async def authorize_request(self, request: PipelineRequest[HTTPRequestType], *scopes: str, **kwargs: Any) -> None:
        """Acquire a token from the credential and authorize the request with it.

        Keyword arguments are passed to the credential's get_token method. The token will be cached and used to
        authorize future requests.

        :param ~azure.core.pipeline.PipelineRequest request: the request
        :param str scopes: required scopes of authentication
        """

        async with self._lock:
            await self._request_token(*scopes, **kwargs)
        bearer_token = cast(Union[AccessToken, AccessTokenInfo], self._token).token
        request.http_request.headers["Authorization"] = "Bearer " + bearer_token

    async def send(
        self, request: PipelineRequest[HTTPRequestType]
    ) -> PipelineResponse[HTTPRequestType, AsyncHTTPResponseType]:
        """Authorize request with a bearer token and send it to the next policy

        :param request: The pipeline request object
        :type request: ~azure.core.pipeline.PipelineRequest
        :return: The pipeline response object
        :rtype: ~azure.core.pipeline.PipelineResponse
        """
        await await_result(self.on_request, request)
        response: PipelineResponse[HTTPRequestType, AsyncHTTPResponseType]
        try:
            response = await self.next.send(request)
        except Exception:
            await await_result(self.on_exception, request)
            raise
        await await_result(self.on_response, request, response)

        if response.http_response.status_code == 401:
            self._token = None  # any cached token is invalid
            if "WWW-Authenticate" in response.http_response.headers:
                request_authorized = await self.on_challenge(request, response)
                if request_authorized:
                    # if we receive a challenge response, we retrieve a new token
                    # which matches the new target. In this case, we don't want to remove
                    # token from the request so clear the 'insecure_domain_change' tag
                    request.context.options.pop("insecure_domain_change", False)
                    try:
                        response = await self.next.send(request)
                    except Exception:
                        await await_result(self.on_exception, request)
                        raise
                    await await_result(self.on_response, request, response)

        return response

    async def on_challenge(
        self,
        request: PipelineRequest[HTTPRequestType],
        response: PipelineResponse[HTTPRequestType, AsyncHTTPResponseType],
    ) -> bool:
        """Authorize request according to an authentication challenge

        This method is called when the resource provider responds 401 with a WWW-Authenticate header.

        :param ~azure.core.pipeline.PipelineRequest request: the request which elicited an authentication challenge
        :param ~azure.core.pipeline.PipelineResponse response: the resource provider's response
        :returns: a bool indicating whether the policy should send the request
        :rtype: bool
        """
        headers = response.http_response.headers
        error = get_challenge_parameter(headers, "Bearer", "error")
        if error == "insufficient_claims":
            encoded_claims = get_challenge_parameter(headers, "Bearer", "claims")
            if not encoded_claims:
                return False
            try:
                padding_needed = -len(encoded_claims) % 4
                claims = base64.urlsafe_b64decode(encoded_claims + "=" * padding_needed).decode("utf-8")
                if claims:
                    token = await self._get_token(*self._scopes, claims=claims)
                    bearer_token = cast(Union["AccessToken", "AccessTokenInfo"], token).token
                    request.http_request.headers["Authorization"] = "Bearer " + bearer_token
                    return True
            except Exception:  # pylint:disable=broad-except
                return False
        return False

    def on_response(
        self,
        request: PipelineRequest[HTTPRequestType],
        response: PipelineResponse[HTTPRequestType, AsyncHTTPResponseType],
    ) -> Optional[Awaitable[None]]:
        """Executed after the request comes back from the next policy.

        :param request: Request to be modified after returning from the policy.
        :type request: ~azure.core.pipeline.PipelineRequest
        :param response: Pipeline response object
        :type response: ~azure.core.pipeline.PipelineResponse
        """

    def on_exception(self, request: PipelineRequest[HTTPRequestType]) -> None:
        """Executed when an exception is raised while executing the next policy.

        This method is executed inside the exception handler.

        :param request: The Pipeline request object
        :type request: ~azure.core.pipeline.PipelineRequest
        """
        # pylint: disable=unused-argument
        return

    def _need_new_token(self) -> bool:
        now = time.time()
        refresh_on = getattr(self._token, "refresh_on", None)
        return not self._token or (refresh_on and refresh_on <= now) or self._token.expires_on - now < 300

    async def _get_token(self, *scopes: str, **kwargs: Any) -> Union["AccessToken", "AccessTokenInfo"]:
        if self._enable_cae:
            kwargs.setdefault("enable_cae", self._enable_cae)

        if hasattr(self._credential, "get_token_info"):
            options: TokenRequestOptions = {}
            # Loop through all the keyword arguments and check if they are part of the TokenRequestOptions.
            for key in list(kwargs.keys()):
                if key in TokenRequestOptions.__annotations__:  # pylint: disable=no-member
                    options[key] = kwargs.pop(key)  # type: ignore[literal-required]

            return await await_result(
                cast(AsyncSupportsTokenInfo, self._credential).get_token_info,
                *scopes,
                options=options,
            )
        return await await_result(
            cast(AsyncTokenCredential, self._credential).get_token,
            *scopes,
            **kwargs,
        )

    async def _request_token(self, *scopes: str, **kwargs: Any) -> None:
        """Request a new token from the credential.

        This will call the credential's appropriate method to get a token and store it in the policy.

        :param str scopes: The type of access needed.
        """
        self._token = await self._get_token(*scopes, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any, IO, Union

from azure.identity import DefaultAzureCredential

from azure.mgmt.cdn import CdnManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-cdn
# USAGE
    python afd_custom_domains_create.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = CdnManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="subid",
    )

    response = client.afd_custom_domains.begin_create(
        resource_group_name="RG",
        profile_name="profile1",
        custom_domain_name="domain1",
        custom_domain={
            "properties": {
                "azureDnsZone": {"id": ""},
                "hostName": "www.someDomain.net",
                "tlsSettings": {"certificateType": "ManagedCertificate", "minimumTlsVersion": "TLS12"},
            }
        },
    ).result()
    print(response)


# x-ms-original-file: specification/cdn/resource-manager/Microsoft.Cdn/stable/2024-02-01/examples/AFDCustomDomains_Create.json
if __name__ == "__main__":
    main()
